# -*- coding: utf-8 -*-
#    Copyright 2005 Spike^ekipS <spikeekips@gmail.com>
#
#       This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import gtk
import Conf

class __Group__ :
	GROUP = None
	WINDOW = None

	def __init__ (self) :
		self.GROUP = gtk.AccelGroup()

	def __del__ (self) :
		if self.WINDOW : self.WINDOW.remove_accel_group(self.GROUP)

	def lock (self) : self.GROUP.lock()
	def unlock (self) : self.GROUP.unlock()

	def addToWindow (self, window) :
		if not isinstance(window, gtk.Window) :
			raise TypeError, _("window must be the instance of gtk.Window.")

		self.WINDOW = window
		self.WINDOW.add_accel_group(self.GROUP)

	def addKey (self, accel_key, accel_mods, callback) :
		if not gtk.accelerator_valid(accel_key, accel_mods) :
			return False

		self.GROUP.connect_group(accel_key, accel_mods, \
			gtk.ACCEL_VISIBLE, callback)

		return True

	def removeKey (self, accel_key, accel_mods) :
		if not gtk.accelerator_valid(accel_key, accel_mods) :
			return False

		return self.GROUP.disconnect_key(accel_key, accel_mods)

class Group (__Group__) :

	UID_ID = dict()
	GCONF_CLIENT = None

	def __init__ (self, gconf_client=None) :
		__Group__.__init__(self)

		if gconf_client :
			self.GCONF_CLIENT = gconf_client

	def addKey (self, uid, accel_key, accel_mods, callback) :
		if self.UID_ID.has_key(uid) : return True

		self.UID_ID[uid] = (accel_key, accel_mods, callback, )

		return __Group__.addKey(self, accel_key, accel_mods, callback)

	def updateKey \
			(self, uid, new_accel_key, new_accel_mods, old_callback=None) :
		if not self.UID_ID.has_key(uid) : return False

		if not old_callback : old_callback = self.UID_ID[uid][2]

		if not self.removeKey(uid) : return False

		return self.addKey(uid, new_accel_key, new_accel_mods, old_callback)

	def removeKey (self, uid) :
		if not self.UID_ID.has_key(uid) : return False

		__accel_key = self.UID_ID[uid][0]
		__accel_mods = self.UID_ID[uid][1]

		del(self.UID_ID[uid])

		return __Group__.removeKey(self, __accel_key, __accel_mods)

	def getData (self, conf_key) :
		if not Conf.LIST_KEYBOARD_SHORTCUT.has_key(conf_key) :
			return (None, None, None, )

		__data = Conf.LIST_KEYBOARD_SHORTCUT[conf_key]
		__callback = __data[1]

		__accel_keys = self.GCONF_CLIENT.getValue(conf_key)
		if not __accel_keys : __accel_keys = __data[0]
		if not __accel_keys : return (None, None, None, )

		__key, __mod = gtk.accelerator_parse(__accel_keys)

		return (__key, __mod, __callback, )

"""
Description
-----------


ChangeLog
---------


Usage
-----


"""

__author__ =  "Spike^ekipS <spikeekips@gmail.com>"
__version__=  "0.1"
__nonsense__ = ""

__file__ = "Accel.py"


