# -*- coding: utf-8 -*-
#	Copyright 2005 Spike^ekipS <spikeekips@gmail.com>
#
#	   This program is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import os, struct, errno, stat, time, fcntl, tty
import gtk

from twisted.internet import defer
from twisted.python import failure
from twisted.conch.ssh import transport, filetransfer, common
from twisted.conch.ssh import transport, common
#import filetransfer

import __OS__

class FileTransferClient (filetransfer.FileTransferClient) :
	WINDOW = None
	CURRENT_DIRECTORY = None

	def __init__ (self, window) :
		filetransfer.FileTransferClient.__init__(self)
		self.WINDOW = window

	def connectionMade (self) :
		self.goHome()
		filetransfer.FileTransferClient.connectionMade(self)

	def goHome (self) :
		self.realPath("").addCallback(self._cbSetInitDir)

	def _cbSetInitDir (self, path) :
		# initial directory path.
		self.CURRENT_DIRECTORY = path
		self.WINDOW.listAllRemote(path)

	def makeConnection(self, transport):
		#self.connected = 1
		#self.transport = transport
		#self.connectionMade()
		filetransfer.FileTransferClient.makeConnection(self, transport)

		#self.WINDOW.toggle_connected(True)
		print "connected"

class FileTransfer :
	"""
	Available commands :
	=====================
		cd path									Change remote directory to 'path'.
		chgrp gid path							Change gid of 'path' to 'gid'.
		chmod mode path						Change mode of 'path' to 'mode'.
		chown uid path							Change uid of 'path' to 'uid'.
		exit										Disconnect from the server.
		get remote-path [local-path]		Get remote file.
		help										Get a list of available commands.
		lcd path									Change local directory to 'path'.
		lls [ls-options] [path]				Display local directory listing.
		lmkdir path								Create local directory.
		ln linkpath targetpath				Symlink remote file.
		lpwd										Print the local working directory.
		ls [-l] [path]							Display remote directory listing.
		mkdir path								Create remote directory.
		progress									Toggle progress bar.
		put local-path [remote-path]		Put local file.
		pwd										Print the remote working directory.
		quit										Disconnect from the server.
		rename oldpath newpath				Rename remote file.
		rmdir path								Remove remote directory.
		rm path									Remove remote file.
		version									Print the SFTP version.
		?											Synonym for 'help'.
	"""

	CURRENT_DIRECTORY = ""
	TRANSFER = None

	def __init__ (self, window) :
		self.WINDOW = window

		self.TRANSFER = FileTransferClient(window)

	def getClient (self) :
		return self.TRANSFER

	###########################################################
	# COMMAND method
	def getRealPath (self, path, isFile=False) :
		if not path.strip().startswith("/") :
			path = os.path.join(self.CURRENT_DIRECTORY, path.strip())

		if not isFile and not path.endswith('/') :
			path += '/'

		return path

	def getAttrs (self, path):
		return self.TRANSFER.getAttrs(self.getRealPath(path))

	def cmd_CD(self, path) :
		d = self.TRANSFER.openDirectory(self.getRealPath(path))
		d.addCallback(self._cbCd)

		return d

	def cmd_CHGRP(self, gid, path) :
		gid = int(gid)
		d = self.TRANSFER.getAttrs(self.getRealPath(path))
		d.addCallback(self._cbSetUsrGrp, path, grp=gid)
		return d

	def cmd_CHMOD (self, mod, path) :
		try :
			mod = int(mod, 8)
		except ValueError :
			self._printFailure(failure.Failure())

		d = self.TRANSFER.setAttrs( \
			self.getRealPath(path), {'permissions':mod} \
		)
		#d.addCallback(_ignore)
		return d

	def cmd_CHOWN(self, uid, path) :
		usr = int(uid)
		d = self.TRANSFER.getAttrs(self.getRealPath(path))
		d.addCallback(self._cbSetUsrGrp, path, usr=uid)
		return d

	def cmd_GET(self, remotefilepath, localfilepath, ProgressBar=None) :

		if os.path.isdir(localfilepath) :
			localfilepath = os.path.join( \
				localfilepath,
				os.path.basename(remotefilepath) \
			)

		try :
			#lf = file(localfilepath, 'w', 0)
			lf = DummyFileRemote(localfilepath)
		except IOError :
			return failure.Failure()

		remotefilepath = self.getRealPath(remotefilepath, "is_file")

		d = self.TRANSFER.openFile( \
			remotefilepath, filetransfer.FXF_READ, {} \
		)
		d.addCallback(self._cbGetOpenFile, lf, ProgressBar)
		d.addErrback(self._ebCloseLf, lf)
		return d

	def cmd_PUT(self, localfilepath, remotefilepath, ProgressBar=None) :

		lf = file(localfilepath, 'r')
		lf = DummyFileLocal(localfilepath)

		d = self.TRANSFER.openFile(remotefilepath, FXF_WRITE | FXF_CREAT, {})

		d.addCallback(self._cbPutOpenFile, lf, ProgressBar)
		d.addErrback(self._ebCloseLf, lf)
		return d

	def cmd_LN(self, source, target) :
		linkpath, targetpath = \
			map(lambda x: \
				os.path.join(self.CURRENT_DIRECTORY, x), \
				(source, target) \
			)

		#return self.TRANSFER.makeLink( \
		#	linkpath, targetpath).addCallback(_ignore)
		return self.TRANSFER.makeLink( \
			linkpath, targetpath)

	def cmd_LS(self, path=None) :
		if not path :
			path = self.CURRENT_DIRECTORY

		d = self._remoteGlob(self.getRealPath(path))
		d.addCallback(self._cbReturnFiles)
		return d

	def cmd_MKDIR(self, path) :
		#return self.TRANSFER.makeDirectory( \
		#	self.getRealPath(path), {}).addCallback(_ignore)
		return self.TRANSFER.makeDirectory(path, {})
			#self.getRealPath(path), {})

	def cmd_RMDIR(self, path) :
		return self.TRANSFER.removeDirectory(self.getRealPath(path))

	def cmd_RM(self, path) :
		return self.TRANSFER.removeFile(self.getRealPath(path, "this isfile"))

	def cmd_RENAME(self, old, new) :
		return self.TRANSFER.renameFile(old, new)

	def cmd_EXIT(self, ignored) :
		self.TRANSFER.transport.loseConnection()

	cmd_QUIT = cmd_EXIT

	def cmd_VERSION(self, ignored) :
		return "SFTP version %i" % self.TRANSFER.version

	def cmd_PWD(self, ignored) :
		return self.CURRENT_DIRECTORY

	def cmd_EXEC(self, rest) :
		shell = pwd.getpwnam(getpass.getuser())[6]

		if rest :
			cmds = ['-c', rest]
			return utils.getProcessOutput(shell, cmds, errortoo=1)
		else :
			os.system(shell)

	#########################################################################
	# MISC handler
	def _cbCd(self, directory) :

		directory.close()
		d = self.TRANSFER.realPath(directory.name)
		d.addCallback(self._cbCurDir)
		return d

	def _cbCurDir(self, path) :
		self.CURRENT_DIRECTORY = path

	def _cbSetUsrGrp(self, attrs, path, usr=None, grp=None) :
		new = {}
		new['uid'] = (usr is not None) and usr or attrs['uid']
		new['gid'] = (grp is not None) and grp or attrs['gid']
		d = self.TRANSFER.setAttrs(path, new)
		#d.addCallback(_ignore)
		return d

	def _ebCloseLf(self, f, lf) :
		lf.close("Don't swap")
		return f

	def _cbGetOpenFile(self, rf, lf, ProgressBar=None) :
		return rf.getAttrs().addCallback(self._cbGetFileSize, rf, lf, ProgressBar)

	def _cbGetFileSize(self, attrs, rf, lf, ProgressBar=None) :
		if not stat.S_ISREG(attrs['permissions']) :
			rf.close()
			lf.close("remove it!")
			return "Can't get non-regular file: %s" % rf.name

		rf.size = attrs['size']

		bufferSize = \
			self.TRANSFER.transport.conn.CONF["options"]['buffersize']
		numRequests = \
			self.TRANSFER.transport.conn.CONF["options"]['requests']

		rf.total = 0.0

		dList = []
		chunks = []
		startTime = time.time()
		ProgressBar.starting(lf, rf, startTime)

		for i in range(numRequests) :
			d = self._cbGetRead('', rf, lf, chunks, 0, bufferSize, ProgressBar)
			dList.append(d)

		dl = defer.DeferredList(dList, fireOnOneErrback=1)
		dl.addCallback(self._cbGetDone, rf, lf)
		return dl

	def _getNextChunk(self, chunks) :
		end = 0
		for chunk in chunks :
			if end == 'eof':
				return # nothing more to get

			if end != chunk[0]:
				i = chunks.index(chunk)
				chunks.insert(i, (end, chunk[0]))
				return (end, chunk[0] - end)

			end = chunk[1]

		bufSize = int( \
			self.TRANSFER.transport.conn.CONF["options"]['buffersize'])
		chunks.append((end, end + bufSize))
		return (end, bufSize)

	IS_STOP_TRANSFER = False
	def _cbGetRead (self, data, rf, lf, chunks, start, size, progressBar=None) :
		if self.IS_STOP_TRANSFER :
			self._cbStopTransfer(rf, lf)
			#raise RuntimeError, "stop this operation."
			if progressBar : progressBar.stop()
			return

		if lf is None :
			if progressBar : progressBar.stop()
			return

		if data and isinstance(data, failure.Failure) :
			reason = data
			reason.trap(EOFError)
			i = chunks.index((start, start + size))
			del chunks[i]
			chunks.insert(i, (start, 'eof'))
		elif data :
			lf.seek(start)
			lf.write(data)

			############################
			# NOTE :
			#  In twisted.conch.ssh.filetransfer, openFile and
			# ClientFile.readChunk() can not handle properly Big size file, so i
			# added this routine and sys.setrecursionlimit(1000*10) for preventing
			# "max recursion depth excedded" error.
			if rf.size > 1000000 :
				time.sleep(0.01)

			if len(data) != size :
				i = chunks.index((start, start + size))
				del chunks[i]
				chunks.insert(i, (start, start + len(data)))

			rf.total += len(data)

		chunk = self._getNextChunk(chunks)
		if not chunk :
			return
		else :
			start, length = chunk

		d = rf.readChunk(start, length)
		d.addBoth( \
			self._cbGetRead, rf, lf, chunks, start, length, progressBar)

		return d

	def stopTransfer (self) :
		self.IS_STOP_TRANSFER = True

	def _cbStopTransfer (self, rf, lf) :
		self.IS_STOP_TRANSFER = False
		rf.close()
		lf.close()

	def _cbGetDone(self, ignored, rf, lf) :
		rf.close()
		lf.close()

		# refresh local file tree.
		self.WINDOW.listAllLocal()
		return "Transferred %s to %s" % (rf.name, lf.name)

	def _remoteGlob(self, fullPath) :
		d = self.TRANSFER.openDirectory(fullPath)
		d.addCallback(self._cbOpenList)

		return d

	def _cbReadFiles (self, result, fullpath, files) :
		dList = []
		for i in result :
			files.extend([os.path.join(fullpath, data[0]), ] + list(i)[1:])
			d = self._cbGetDirectory(i, fullpath, files)
			dList.append(d)

		dl = defer.DeferredList(dList)
		dl.addCallback(self._cbReadFiles, rf, lf)
		return dl

	def _cbOpenList(self, directory) :
		files = []
		d = directory.read()
		d.addBoth(self._cbReadFile, files, directory)
		return d

	def _ebNotADirectory(self, reason, path, glob) :
		d = self.TRANSFER.openDirectory(path)
		d.addCallback(self._cbOpenList, glob)
		return d

	def _cbReadFile(self, files, l, directory) :
		if not isinstance(files, failure.Failure) :
			l.extend(files)

			d = directory.read()
			d.addBoth(self._cbReadFile, l, directory)
			return d
		else :
			reason = files
			reason.trap(EOFError)
			directory.close()
			return l

	def _cbPutOpenFile(self, rf, lf, ProgressBar=None) :

		startTime = time.time()
		ProgressBar.starting(lf, rf, startTime)

		numRequests = self.TRANSFER.transport.conn.CONF["options"]['requests']
		dList = []
		chunks = []
		for i in range(numRequests) :
			d = self._cbPutWrite(None, rf, lf, chunks, ProgressBar)
			if d :
				dList.append(d)

		dl = defer.DeferredList(dList, fireOnOneErrback=1)
		dl.addCallback(self._cbPutDone, rf, lf)
		return dl

	def _cbPutWrite(self, ignored, rf, lf, chunks, ProgressBar=None) :
		chunk = self._getNextChunk(chunks)
		start, size = chunk
		lf.seek(start)
		data = lf.read(size)
		lf.total += len(data)

		if data :
			d = rf.writeChunk(start, data)
			d.addCallback(self._cbPutWrite, rf, lf, chunks, ProgressBar)
			return d
		else :
			return

	def _cbPutDone(self, ignored, rf, lf) :
		lf.close()
		rf.close()

		return 'Transferred %s to %s' % (lf.name, rf.name)

	def _cbReturnFiles (self, files) :
		return files

	def _cbDisplayFiles(self, files) :
		files.sort()

		files = [f for f in files if not f[0].startswith('.')]
		lines = [f[1] for f in files]

		if not lines :
			return None
		else :
			return '\n'.join(lines)

	def _printFailure(self, f):
		e = f.trap( \
			NotImplementedError,\
			filetransfer.SFTPError, \
			OSError, \
			IOError \
		)

		if e == NotImplementedError :
			self.transport.write(self.cmd_HELP(''))
		elif e == filetransfer.SFTPError :
			self.transport.write( \
				"remote error %i: %s\n" % (f.value.code, f.value.message))
		elif e in (OSError, IOError) :
			self.transport.write( \
				"local error %i: %s\n" % (f.value.errno, f.value.strerror))

import tempfile, shutil, os
class DummyFileLocal (file) :
	total = 0
	size = 0
	def __init__ (self, filename, mode="r", buffering=0) :
		file.__init__(self, filename, mode, buffering)
		self.size = os.fstat(self.fileno())[6]

	def close (self, isRemove=False) :
		file.close(self)

class DummyFileRemote (file) :
	"""
	store data in temporary file.
	"""

	NAME_REAL = ""
	NAME_TEMPORARY = ""

	def __init__ (self, filename) :

		self.NAME_REAL = filename

		self.NAME_TEMPORARY = \
			tempfile.mktemp(prefix="vee-scpfile-", suffix=".tmp")

		file.__init__(self, self.NAME_TEMPORARY, "w")

	def close (self, isRemove=False) :
		file.close(self)
		if isRemove :
			os.remove(self.NAME_TEMPORARY)
		else :
			self.swap()

	def swap (self) :
		shutil.move(self.NAME_TEMPORARY, self.NAME_REAL)
		#shutil.copy(self.NAME_TEMPORARY, self.NAME_REAL)

def _ignore (*args) : pass

"""
DATA
"""
FILEXFER_ATTR_ACMODTIME   = filetransfer.FILEXFER_ATTR_ACMODTIME
FILEXFER_ATTR_EXTENDED    = filetransfer.FILEXFER_ATTR_EXTENDED
FILEXFER_ATTR_OWNERGROUP  = filetransfer.FILEXFER_ATTR_OWNERGROUP
FILEXFER_ATTR_PERMISSIONS = filetransfer.FILEXFER_ATTR_PERMISSIONS
FILEXFER_ATTR_SIZE        = filetransfer.FILEXFER_ATTR_SIZE
FILEXFER_TYPE_DIRECTORY   = filetransfer.FILEXFER_TYPE_DIRECTORY
FILEXFER_TYPE_REGULAR     = filetransfer.FILEXFER_TYPE_REGULAR
FILEXFER_TYPE_SPECIAL     = filetransfer.FILEXFER_TYPE_SPECIAL
FILEXFER_TYPE_SYMLINK     = filetransfer.FILEXFER_TYPE_SYMLINK
FILEXFER_TYPE_UNKNOWN     = filetransfer.FILEXFER_TYPE_UNKNOWN
FXF_APPEND                = filetransfer.FXF_APPEND
FXF_CREAT                 = filetransfer.FXF_CREAT
FXF_EXCL                  = filetransfer.FXF_EXCL
FXF_READ                  = filetransfer.FXF_READ
FXF_TEXT                  = filetransfer.FXF_TEXT
FXF_TRUNC                 = filetransfer.FXF_TRUNC
FXF_WRITE                 = filetransfer.FXF_WRITE
FXP_ATTRS                 = filetransfer.FXP_ATTRS
FXP_CLOSE                 = filetransfer.FXP_CLOSE
FXP_DATA                  = filetransfer.FXP_DATA
FXP_EXTENDED              = filetransfer.FXP_EXTENDED
FXP_EXTENDED_REPLY        = filetransfer.FXP_EXTENDED_REPLY
FXP_FSETSTAT              = filetransfer.FXP_FSETSTAT
FXP_FSTAT                 = filetransfer.FXP_FSTAT
FXP_HANDLE                = filetransfer.FXP_HANDLE
FXP_INIT                  = filetransfer.FXP_INIT
FXP_LSTAT                 = filetransfer.FXP_LSTAT
FXP_MKDIR                 = filetransfer.FXP_MKDIR
FXP_NAME                  = filetransfer.FXP_NAME
FXP_OPEN                  = filetransfer.FXP_OPEN
FXP_OPENDIR               = filetransfer.FXP_OPENDIR
FXP_READ                  = filetransfer.FXP_READ
FXP_READDIR               = filetransfer.FXP_READDIR
FXP_READLINK              = filetransfer.FXP_READLINK
FXP_REALPATH              = filetransfer.FXP_REALPATH
FXP_REMOVE                = filetransfer.FXP_REMOVE
FXP_RENAME                = filetransfer.FXP_RENAME
FXP_RMDIR                 = filetransfer.FXP_RMDIR
FXP_SETSTAT               = filetransfer.FXP_SETSTAT
FXP_STAT                  = filetransfer.FXP_STAT
FXP_STATUS                = filetransfer.FXP_STATUS
FXP_SYMLINK               = filetransfer.FXP_SYMLINK
FXP_VERSION               = filetransfer.FXP_VERSION
FXP_WRITE                 = filetransfer.FXP_WRITE
FX_BAD_MESSAGE            = filetransfer.FX_BAD_MESSAGE
FX_CONNECTION_LOST        = filetransfer.FX_CONNECTION_LOST
FX_EOF                    = filetransfer.FX_EOF
FX_FAILURE                = filetransfer.FX_FAILURE
FX_NO_CONNECTION          = filetransfer.FX_NO_CONNECTION
FX_NO_SUCH_FILE           = filetransfer.FX_NO_SUCH_FILE
FX_OK                     = filetransfer.FX_OK
FX_OP_UNSUPPORTED         = filetransfer.FX_OP_UNSUPPORTED
FX_PERMISSION_DENIED      = filetransfer.FX_PERMISSION_DENIED


"""
Description
-----------


ChangeLog
---------


Usage
-----


"""

__author__ =  "Spike^ekipS <spikeekips@gmail.com>"
__version__=  "0.1"
__nonsense__ = ""

__file__ = "FileTransfer.py"


