# -*- coding: utf-8 -*-
#    Copyright (C) 2005 Spike^ekipS <spikeekips@gmail.com>
#
#    http://vee.kldp.net
#
#       This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import os, gconf, time
import Window

class Client :

	CLIENT = None
	APP_PATH = None

	def __init__ (self, path) :

		self.CLIENT = gconf.client_get_default();

		self.setPath(path)

		if not self.addDir(path) :
			raise RuntimeError, "%s does not exists." % path
		else :
			self.APP_PATH = path

	def addDir (self, path) :
		try :
			self.CLIENT.add_dir( \
				os.path.dirname(path), gconf.CLIENT_PRELOAD_NONE)
		except :
			return False
		else :
			return True

	def setPath (self, path, make_new=False) :
		path = os.path.dirname(path)

		if not self.CLIENT.dir_exists(path) :
			if make_new :
				return self.create_new(path)
			else :
				raise ValueError, "%s does not exists." % path

		else :
			return True

	def generatePath (self, key) :
		return os.path.join(self.APP_PATH, key)

	def getValueByConf (self, conf_key) :
		return self.getValue(conf_key)

	def getValueType (self, key) :
		key = self.generatePath(key)
		if not self.CLIENT.dir_exists(os.path.dirname(key)) :
			return None

		try :
			return self.CLIENT.get(key).type
		except AttributeError :
			return None

	def getValue (self, key, defaultValue=None) :

		key = self.generatePath(key)
		if not self.CLIENT.dir_exists(os.path.dirname(key)) :
			return defaultValue

		__value = self.CLIENT.get(key)

		__retval = self.getValueFromValue(__value)

		if __retval is None :
			__retval = defaultValue

		return __retval

	def getValueFromValue (self, GConfValue) :
		if not isinstance(GConfValue, gconf.Value) :
			return None

		__type = GConfValue.type

		if __type == gconf.VALUE_INT :
			__v = GConfValue.get_int()
		elif __type == gconf.VALUE_BOOL :
			__v = GConfValue.get_bool()
		elif __type == gconf.VALUE_STRING :
			__v = GConfValue.get_string()
		elif __type == gconf.VALUE_LIST :
			__v = GConfValue.get_list()
		elif __type == gconf.VALUE_FLOAT :
			__v = GConfValue.get_float()
		else :
			__v = None

		return __v

	def setValue (self, key, value, typeValue=None, set_force=False) :

		key = self.generatePath(key)

		if not self.CLIENT.dir_exists(os.path.dirname(key)) :
			if set_force :
				if not self.create_new(os.path.dirname(key)) :
					return False
			else :
				return False

		# get value type.
		if not typeValue :
			typeValue = self.getValueType(key)

		if typeValue in ("int", gconf.VALUE_INT, ) :
			self.CLIENT.set_int(key, value)
		elif typeValue in ("bool", gconf.VALUE_BOOL, ) :
			self.CLIENT.set_bool(key, value)
		elif typeValue in ("float", gconf.VALUE_FLOAT, ) :
			self.CLIENT.set_float(key, value)
		elif typeValue in ("string", gconf.VALUE_STRING, ) :
			self.CLIENT.set_string(key, value)
		elif typeValue in ("int_list", gconf.VALUE_LIST, ) :
			return self.set_list("int", key, value, set_force)
		else :
			return False

		return True

	def getList (self, typeValue, key) :

		if typeValue == "int" :
			__type = gconf.VALUE_INT
		elif typeValue == "bool" :
			__type = gconf.VALUE_BOOL
		elif typeValue == "string" :
			__type = gconf.VALUE_STRING
		else :
			return False

		return self.CLIENT.get_list(key, __type)

	def setList (self, typeValue, key, list_value, set_force=False) :
		if not self.CLIENT.dir_exists(os.path.dirname(key)) :
			if set_force :
				if not self.create_new(os.path.dirname(key)) :
					return False
			else :
				return False

		if type(list_value) is not (list or tuple) :
			return False

		if typeValue == "int" :
			__type = gconf.VALUE_INT
		elif typeValue == "bool" :
			__type = gconf.VALUE_BOOL
		elif typeValue == "string" :
			__type = gconf.VALUE_STRING
		else :
			return False

		return self.CLIENT.set_list(key, __type, list_value)

	def getValueByPath (self, key_path) :
		if not self.CLIENT.dir_exists(os.path.dirname(key_path)) :
			return defaultValue

		__value = self.CLIENT.get(key_path)
		__retval = self.getValueFromValue(__value)

		if __retval is None :
			__retval = defaultValue

		return __retval

NOTIFICATIONS = { \
	"filename_to_insert_as_new_tab_in_vim" : { \
		"gconf-key" : "filename_to_insert_as_new_tab_in_vim", \
		"callback" : "on_insert_vim", \
	}, \
	"signal_message" : { \
		"gconf-key" : "signal_message", \
		"callback" : "on_show_signal_message", \
	}, \
}

import Conf, func
class Notify :

	NOTIFY_ID = dict()
	GCONF_CLIENT = None

	INSTANCE_VEE = None

	def __init__ (self, gconf_client, vee) :

		self.GCONF_CLIENT = gconf_client
		self.INSTANCE_VEE = vee

	def add (self) :

		for i, j in NOTIFICATIONS.items() :
			self.NOTIFY_ID[i] = self.GCONF_CLIENT.CLIENT.notify_add( \
					os.path.join(Conf.GCONF_APP_DIR, j["gconf-key"]), \
					getattr(self, j["callback"]) \
			)

	def __setDefault (self, entry) :
		self.GCONF_CLIENT.setValue(os.path.basename(entry.get_key()), "")

	def on_show_signal_message (self, client, parm1, entry, parm3) :
		__data = self.GCONF_CLIENT.getValueFromValue(entry.get_value())
		try :
			__uid, __message, __type = __data.split("@@@")
		except ValueError :
			return

		if not __uid : return
		if self.INSTANCE_VEE.UID != __uid : return

		self.__setDefault(entry)

		# error dialog
		Window.SignalMessage(__message, __type)

	def on_insert_vim (self, client, parm1, entry, parm3) :
		__data = self.GCONF_CLIENT.getValueFromValue(entry.get_value())

		(__uid, __filenames) = func.parse_argument_of_veet(__data)

		if not __uid : return
		if self.INSTANCE_VEE.UID != __uid : return

		for i in __filenames :
			self.INSTANCE_VEE.addPage("vim", filename=i)

		self.__setDefault(entry)

def verify (forceInstall=False) :
	"""
	Check whether key and it's type is correct or not, if not insert
	new schema.
	"""

	if forceInstall :
		install_schema()
	else :
		try :
			GCONF_CLIENT = Client(Conf.GCONF_APP_DIR)
		except :
			return False
		else :
			return True

def install_schema () :

	print ">> Installing Schema file, %s" % Conf.FILE_GCONF_SCHMEA
	a, b, c = \
		os.popen3("gconftool-2 --install-schema-file=%s" % \
			Conf.FILE_GCONF_SCHMEA)

	# some weird problems, without b.readlines(), c.readlines() hangs up.
	b.readlines()

	if len(c.readlines()) > 0 :
		print ">> Failed to install Schema file."
		return False

	print ">> Install Schema file."
	print ""

	return True

"""
Description
-----------


ChangeLog
---------


Usage
-----


"""

__author__ =  "Spike^ekipS <spikeekips@gmail.com>"
__version__=  "0.1"
__nonsense__ = ""

__file__ = "GConf.py"

