# -*- coding: utf-8 -*-
#	Copyright (C) 2005 Spike^ekipS <spikeekips@gmail.com>
#
#	http://vee.kldp.net
#
#	   This program is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from optparse import OptionParser

import Conf__, Conf, i18n
_ = i18n.gettext()

try :
	from twisted.conch.client import options as TwistedConchClientOptions
except ImportError :
	pass
else :
	class SSHClient (TwistedConchClientOptions.ConchOptions) :

		synopsis = \
	"""Usage:   sshclient.py [options] host [command]
	"""

		optParameters = [ \
			['escape', 'e', '~'], \
			['localforward', 'L', None, 'listen-port:host:port   Forward local port to remote address'], \
			['remoteforward', 'R', None, 'listen-port:host:port   Forward remote port to local address'], \
			['uid', 'U', None, 'uid for VEE'], \
		]

		optFlags = [ \
			['null', 'n', 'Redirect input from /dev/null.'], \
			['fork', 'f', 'Fork to background after authentication.'], \
			['tty', 't', 'Tty; allocate a tty even if command is given.'], \
			['notty', 'T', 'Do not allocate a tty.'], \
			['noshell', 'N', 'Do not execute a shell or command.'], \
			['subsystem', 's', 'Invoke command (mandatory) as SSH2 subsystem.'], \
		]

		#zsh_altArgDescr = {"foo":"use this description for foo instead"}
		#zsh_multiUse = ["foo", "bar"]
		#zsh_mutuallyExclusive = [("foo", "bar"), ("bar", "baz")]
		#zsh_actions = {"foo":'_files -g "*.foo"', "bar":"(one two three)"}
		zsh_actionDescr = { \
			"localforward":"listen-port:host:port", \
			"remoteforward":"listen-port:host:port" \
		}
		zsh_extras = ["*:command: "]

		localForwards = []
		remoteForwards = []

		def opt_escape(self, esc):
			"Set escape character; ``none'' = disable"
			if esc == 'none':
				self['escape'] = None
			elif esc[0] == '^' and len(esc) == 2:
				self['escape'] = chr(ord(esc[1])-64)
			elif len(esc) == 1:
				self['escape'] = esc
			else:
				sys.exit("Bad escape character '%s'." % esc)

		def opt_localforward(self, f):
			"Forward local port to remote address (lport:host:port)"
			localPort, remoteHost, remotePort = f.split(':') # doesn't do v6 yet
			localPort = int(localPort)
			remotePort = int(remotePort)
			self.localForwards.append((localPort, (remoteHost, remotePort)))

		def opt_remoteforward(self, f):
			"""Forward remote port to local address (rport:host:port)"""
			remotePort, connHost, connPort = f.split(':') # doesn't do v6 yet
			remotePort = int(remotePort)
			connPort = int(connPort)
			self.remoteForwards.append((remotePort, (connHost, connPort)))

		def parseArgs(self, host="", *command):
			if host.find("@") == -1 :
				self['host'] = host
			else :
				try :
					(__user, __host, ) = host.split("@")
				except ValueError :
					__s = host.split("@")
					(__user, __host, ) = (__s[:1], __s[1:],)

				self['host'] = __host
				self['user'] = __user

			self['command'] = ' '.join(command)

		def opt_user (self, user="") :
			if not user.strip() :
				user = os.getenv("USER")

			self["user"] = user.strip()

def __OPTIONS__ (options, usage, version="%prog 1.0") :
	parser = OptionParser(version=version)

	parser.usage = usage

	for i, j in options.items() :
		parser.add_option(i, j[0], **j[1])

	return parser

def VEE () :
	version = "%%prog %s" % Conf__.INFO["version"]
	usage = "%prog [options]"
	return __OPTIONS__(Conf.OPTIONS_VEE, usage, version)

def VEET () :
	version = "%prog 1.0"
	usage = "%prog [options]"
	return __OPTIONS__(Conf.OPTIONS_VEET, usage, version)

def parse_argument_of_ssh_options (data) :
	__a = xmlrpclib.loads(data)

	if len(__a[0]) < 2 :
		__uid = None
	else :
		__uid = __a[0][1]

	__data = eval(__a[0][0])

	if type(__data) is not dict : return (None, None)

	return (__uid, __data)

def parse_argument_of_veet (data) :
	__a = xmlrpclib.loads(data)

	if len(__a[0]) < 2 :
		__uid = None
	else :
		__uid = __a[0][1]

	__data = __a[0][0][1:]

	return (__uid, __data)


"""
Description
-----------


ChangeLog
---------


Usage
-----


"""

__author__ =  "Spike^ekipS <spikeekips@gmail.com>"
__version__=  "0.1"
__nonsense__ = ""

__file__ = "Options.py"


