# -*- coding: utf-8 -*-
#	Copyright 2005 Spike^ekipS <spikeekips@gmail.com>
#
#	   This program is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import sre, tempfile
import gtk, gobject, time, os, struct, tty, fcntl, stat, gnomevfs, shutil
try :
	from twisted.internet import defer
	from twisted.python import failure
except ImportError :
	USE_TWISTED = False
else :
	USE_TWISTED = True
	import FileBrowser

import Handler, WIDGET, func, Conf__, Conf, __TREEVIEW__, \
	GNOME, __XML__, __OS__, i18n, Window

_ = i18n.gettext()

class WithGNOME :

	GNOME_ICON = None
	ICONS = dict()

	def generateIcon (self) :
		self.GNOME_ICON = GNOME.Icon()

		for __type, __icons in Conf.ICONS.items() :
			self.ICONS[__type] = self.__getIconPixbuf(__icons)

	def __getIconPixbuf (self, icons=list()) :
		__icon = None
		for i in icons :
			__icon = self.GNOME_ICON.getPixbufByIcon(i)
			if __icon : break

		if not __icon :
			__icon = self.GNOME_ICON.getNone()

		return __icon

class TreeViewWithMultipleItemSelectionDND :

	ROWS_CURRENT_SELECTED = list()
	IS_MULTIPLE_SELECTED = False
	IS_DND_STARTED = False

	def __init__ (self) :
		self.enableDND()

	def enableDND (self) :
		self.WIDGET_TREEVIEW.get_selection().set_mode(gtk.SELECTION_SINGLE)

		self.WIDGET_TREEVIEW.enable_model_drag_source( \
			gtk.gdk.BUTTON1_MASK, \
			Conf.DND_TARGET_TABLE_SSH_FILEBROWSER, \
			gtk.gdk.ACTION_COPY \
		)

		self.WIDGET_TREEVIEW.enable_model_drag_dest( \
			Conf.DND_TARGET_TABLE_SSH_FILEBROWSER, gtk.gdk.ACTION_DEFAULT)

		self.WIDGET_TREEVIEW.drag_source_set( \
			gtk.gdk.BUTTON1_MASK, \
			Conf.DND_TARGET_TABLE_SSH_FILEBROWSER, \
			gtk.gdk.ACTION_COPY \
		)

		self.WIDGET_TREEVIEW.get_selection().set_mode(gtk.SELECTION_MULTIPLE)

		self.WIDGET_TREEVIEW.connect("motion-notify-event", \
			self.on_motion_notify_event)
		self.WIDGET_TREEVIEW.connect("button-press-event", \
			self.on_button_press_event)
		self.WIDGET_TREEVIEW.connect_after("button-release-event", \
			self.on_button_release_event)
		self.WIDGET_TREEVIEW.connect("key-press-event", \
			self.on_key_press_event)
		self.WIDGET_TREEVIEW.connect("key-release-event", \
			self.on_key_release_event)

		self.WIDGET_TREEVIEW.connect("drag-begin", self.on_drag_begin)
		self.WIDGET_TREEVIEW.connect("drag-end", self.on_drag_end)

		self.WIDGET_TREEVIEW.connect("drag_data_get", self.on_drag_data_get_data)
		self.WIDGET_TREEVIEW.connect("drag_data_received", self.on_drag_data_received_data)

	def on_drag_begin (self, widget, context) :
		self.WIDGET_TREEVIEW.drag_dest_unset()

	def on_drag_end (self, widget, context) :
		# re-store the selection
		selection = self.WIDGET_TREEVIEW.get_selection()
		if len(self.ROWS_CURRENT_SELECTED) > 1 :
			selection.unselect_all()
			for i in self.ROWS_CURRENT_SELECTED :
				selection.select_path(i)

			self.IS_MULTIPLE_SELECTED = True

		self.WIDGET_TREEVIEW.enable_model_drag_dest( \
			Conf.DND_TARGET_TABLE_SSH_FILEBROWSER, gtk.gdk.ACTION_DEFAULT)

	def on_motion_notify_event (self, treeview, event) :
		if self.IS_MULTIPLE_SELECTED and self.IS_DND_STARTED :
			self.IS_DND_STARTED = False

	def on_button_release_event (self, treeview, event) :
		try :
			path, column, x, y = self.WIDGET_TREEVIEW.get_path_at_pos(int(event.x), int(event.y))
		except TypeError :
			return

		selection = self.WIDGET_TREEVIEW.get_selection()
		if self.IS_MULTIPLE_SELECTED :
			selection.unselect_all()
			selection.select_path(path)
			self.IS_MULTIPLE_SELECTED = False
			return

		self.ROWS_CURRENT_SELECTED = selection.get_selected_rows()[1]

		if len(self.ROWS_CURRENT_SELECTED) < 2 :
			self.IS_MULTIPLE_SELECTED = False
		else :
			self.IS_MULTIPLE_SELECTED = True

		return False

	def on_button_press_event (self, treeview, event) :

		if event.button == 1 :
			selection = self.WIDGET_TREEVIEW.get_selection()
			try :
				path, column, x, y = self.WIDGET_TREEVIEW.get_path_at_pos(int(event.x), int(event.y))
			except TypeError :
				return

			if self.IS_MULTIPLE_SELECTED :
				self.IS_DND_STARTED = True

				return True
			else :
				self.IS_DND_STARTED = False

		return False

	def on_key_press_event (self, treeview, event) :
		__keyname = func.get_keyname(event)

		if __keyname == "Return" :
			__iters = self.get_selected()[1]
			if len(__iters) > 1 :
				return True

			if isinstance(__iters[0], gtk.TreeIter) :
				__iter = __iters[0]
			else :
				__iter = self.get_iter(__iters[0])

			__name = self.getName(__iter)
			__perms = self.getPermissions(__iter)

			if __OS__.is_directory_by_st_mode(int(__perms)) : # if this is directory
				return False
			else : # if this is not a directory, view file.
				return True

	def on_key_release_event (self, treeview, event) :

		__retval = TreeViewFileList.on_key_release_event(self, event)
		if __retval == True : return __retval

		selection = self.WIDGET_TREEVIEW.get_selection()
		self.ROWS_CURRENT_SELECTED = selection.get_selected_rows()[1]

		if len(self.ROWS_CURRENT_SELECTED) < 2 :
			self.IS_MULTIPLE_SELECTED = False
		else :
			self.IS_MULTIPLE_SELECTED = True

		return False

	def on_drag_data_get_data (self, treeview, context, selection, target_id, etime) :
		"""
		NOT IMPLEMENTED
		"""
		pass

	def on_drag_data_received_data (self, treeview, context, x, y, selection, info, etime) :
		"""
		NOT IMPLEMENTED
		"""
		pass

class TreeViewFileList (TreeViewWithMultipleItemSelectionDND) :

	FILE_TRANSFER = None

	def __init__ (self) :
		self.generateIcon()
		TreeViewWithMultipleItemSelectionDND.__init__(self)

	def set_editable (self, iter) :
		__path = self.get_path(iter)

		__column = self.WIDGET_TREEVIEW.get_column(0)
		cell = __column.get_cell_renderers()[1]
		cell.set_property("editable", True)

		self.WIDGET_TREEVIEW.set_cursor(__path, __column, True)

	def getPermissions (self, iter) :
		if not iter : return None

		return self.WIDGET_TREESTORE.get_value(iter, \
			Conf.TREEVIEW_FILELIST_PANEL_COLUMN_PERMISSION_BIN)

	def getName (self, iter) :
		if not iter : return None

		return self.WIDGET_TREESTORE.get_value(iter, \
			Conf.TREEVIEW_FILELIST_PANEL_COLUMN_NAME)

	def create_frame (self) :
		# create a CellRenderers to render the data
		__cell1 = gtk.CellRendererPixbuf()
		__cell2 = gtk.CellRendererText()
		__cell3 = gtk.CellRendererText()
		__cell4 = gtk.CellRendererText()
		__cell5 = gtk.CellRendererText()

		__cell2.connect("edited", self.on_rename_edited)
		__cell2.connect("editing-canceled", self.on_editing_canceled)

		# set property.
		__cell1.set_property( \
			"width", self.ICONS["__default__"].get_property("width"))
		__cell1.set_property("xalign", 0.5)
		__cell2.set_property("xalign", 0.0)
		__cell3.set_property("xalign", 1.0)
		__cell4.set_property("xalign", 0.0)
		__cell5.set_property("xalign", 0.0)

		self.tvcolumn1 = gtk.TreeViewColumn(_("Name"))
		self.tvcolumn2 = gtk.TreeViewColumn(_("Size"))
		self.tvcolumn3 = gtk.TreeViewColumn(_("Permission"))
		self.tvcolumn4 = gtk.TreeViewColumn(_("Modified Time"))
		self.tvcolumn1.set_expand(True)

		self.WIDGET_TREESTORE = gtk.ListStore( \
			gobject.TYPE_OBJECT, gobject.TYPE_STRING, \
			gobject.TYPE_STRING, gobject.TYPE_STRING, \
			gobject.TYPE_STRING, gobject.TYPE_STRING, \
			gobject.TYPE_STRING, \
		)

		self.WIDGET_TREEVIEW.set_model(self.WIDGET_TREESTORE)

		# add columns to treeview
		self.WIDGET_TREEVIEW.append_column(self.tvcolumn1)
		self.WIDGET_TREEVIEW.append_column(self.tvcolumn2)
		self.WIDGET_TREEVIEW.append_column(self.tvcolumn3)
		self.WIDGET_TREEVIEW.append_column(self.tvcolumn4)

		self.tvcolumn1.pack_start(__cell1, False)
		self.tvcolumn1.pack_start(__cell2, False)
		self.tvcolumn2.pack_start(__cell3, False)
		self.tvcolumn3.pack_start(__cell4, False)
		self.tvcolumn4.pack_start(__cell5, False)

		self.tvcolumn1.set_attributes( \
			__cell1, pixbuf=Conf.TREEVIEW_FILELIST_PANEL_COLUMN_ICON)
		self.tvcolumn1.set_attributes( \
			__cell2, text=Conf.TREEVIEW_FILELIST_PANEL_COLUMN_NAME)
		self.tvcolumn2.set_attributes( \
			__cell3, text=Conf.TREEVIEW_FILELIST_PANEL_COLUMN_SIZE)
		self.tvcolumn3.set_attributes( \
			__cell4, text=Conf.TREEVIEW_FILELIST_PANEL_COLUMN_PERMISSION)
		self.tvcolumn4.set_attributes( \
			__cell5, text=Conf.TREEVIEW_FILELIST_PANEL_COLUMN_CREATED_TIME)

		self.tvcolumn1.set_resizable(True)
		self.tvcolumn2.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
		self.tvcolumn3.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
		self.tvcolumn4.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)

	def insert_row (self) :
		pass

	###################################################################
	# EVENT HANDLER
	def on_key_release_event (self, event) :
		__keyname = func.get_keyname(event)

		# remove
		if __keyname == "Delete" :
			__store, __paths = self.get_selected()
			if len(__paths) < 1 : return

			self.on_remove(__paths)
			return True

		# rename
		elif __keyname == "F2" :
			__store, __paths = self.get_selected()
			if len(__paths) > 1 : return

			self.set_editable(self.get_iter(__paths[0]))
			return True

		# go parent directory
		elif __keyname == "BackSpace" :
			self.on_directory_up()
			return True

		# cursor moves.
		elif __keyname in ["<Shift>Down", "<Shift>Up", ] :
			return False

		# refresh file list.
		elif __keyname in ["<Control>R", "F5", ] :
			self.on_refresh()
			return True
		else :
			return False

	def on_directory_up (self) :
		"""
		NOT IMPLEMENTED
		"""
		pass

class FileTransferStatus (__TREEVIEW__.Core, WithGNOME) :

	FILE_TRANSFER = None
	PARENT = None

	HANDLER_ID_CURSOR_CHANGED = None
	FILETRANSFER = None
	FILETRANSFERQUEUE = None

	def __init__ (self, xml, queue, filetransfer, **kwargs) :
		PROPERTIES = { \
			"enable-search" : True, \
			"fixed-height-mode" : False, \
			"headers-clickable" : False, \
			"headers-visible" : True, \
			"hover-expand" : False, \
			"hover-selection" : False, \
			"reorderable" : False, \
			"rules-hint" : True, \
		}

		self.FILETRANSFER = filetransfer

		self.generateIcon()

		__TREEVIEW__.Core.__init__( \
			self, xml, "treeview-filetransfer-status", properties=PROPERTIES)

		self.WIDGET_TREEVIEW.get_selection().set_mode(gtk.SELECTION_MULTIPLE)

		self.FILETRANSFERQUEUE = queue
		self.FILETRANSFERQUEUE.TREESTORE = self.WIDGET_TREESTORE

		# initiate filetransferQueue
		self.FILETRANSFERQUEUE = self.FILETRANSFER.FILETRANSFERQUEUE
		self.FILETRANSFERQUEUE.startRunning()

	def getCurrentSelectedUID (self) :
		__paths = self.get_selected()[1]
		__uids = list()
		for i in __paths :
			__uids.append(self.getUID(self.get_iter(i)))

		return __uids

	def getUID (self, iter) :
		if not iter : return None

		return self.WIDGET_TREESTORE.get_value(iter, \
			Conf.TREEVIEW_FILE_TRANSFER_STATUS_PANEL_COLUMN_UID)

	def create_frame (self) :
		# create a CellRenderers to render the data
		__cell0 = gtk.CellRendererPixbuf()
		__cell1 = gtk.CellRendererPixbuf()
		__cell2 = gtk.CellRendererText()
		__cell3 = gtk.CellRendererText()
		__cell4 = gtk.CellRendererText()
		__cell5 = gtk.CellRendererText()
		__cell6 = gtk.CellRendererText()
		__cell7 = gtk.CellRendererText()

		# set property.
		__cell0.set_property( \
			"width", self.ICONS["up"].get_property("width"))
		__cell1.set_property( \
			"width", self.ICONS["__default__"].get_property("width"))
		__cell1.set_property("xalign", 0.0)
		__cell2.set_property("xalign", 0.0)
		__cell3.set_property("xalign", 0.0)
		__cell4.set_property("xalign", 1.0)
		__cell5.set_property("xalign", 1.0)
		__cell6.set_property("xalign", 1.0)
		__cell7.set_property("xalign", 0.0)

		self.tvcolumn0 = gtk.TreeViewColumn()
		self.tvcolumn1 = gtk.TreeViewColumn(_("Remote Filename"))
		self.tvcolumn2 = gtk.TreeViewColumn(_("Size"))
		self.tvcolumn3 = gtk.TreeViewColumn(_("TimeLeft"))
		self.tvcolumn4 = gtk.TreeViewColumn(_("%"))
		self.tvcolumn5 = gtk.TreeViewColumn(_("Speed"))
		self.tvcolumn1.set_expand(True)

		self.WIDGET_TREESTORE = gtk.ListStore( \
			gobject.TYPE_OBJECT, \
			gobject.TYPE_OBJECT, gobject.TYPE_STRING, \
			gobject.TYPE_STRING, gobject.TYPE_STRING, \
			gobject.TYPE_STRING, gobject.TYPE_STRING, \
			gobject.TYPE_STRING, gobject.TYPE_STRING, \
		)

		self.WIDGET_TREEVIEW.set_model(self.WIDGET_TREESTORE)

		# add columns to treeview
		self.WIDGET_TREEVIEW.append_column(self.tvcolumn0)
		self.WIDGET_TREEVIEW.append_column(self.tvcolumn1)
		self.WIDGET_TREEVIEW.append_column(self.tvcolumn2)
		self.WIDGET_TREEVIEW.append_column(self.tvcolumn3)
		self.WIDGET_TREEVIEW.append_column(self.tvcolumn4)
		self.WIDGET_TREEVIEW.append_column(self.tvcolumn5)

		self.tvcolumn0.pack_start(__cell0, False)
		self.tvcolumn1.pack_start(__cell1, False)
		self.tvcolumn1.pack_start(__cell2, True)
		self.tvcolumn2.pack_start(__cell7, True)
		self.tvcolumn3.pack_start(__cell4, False)
		self.tvcolumn4.pack_start(__cell5, False)
		self.tvcolumn5.pack_start(__cell6, False)

		self.tvcolumn0.set_attributes( \
			__cell0, pixbuf=Conf.TREEVIEW_FILE_TRANSFER_STATUS_PANEL_COLUMN_DIRECTION)
		self.tvcolumn1.set_attributes( \
			__cell1, pixbuf=Conf.TREEVIEW_FILE_TRANSFER_STATUS_PANEL_COLUMN_ICON)
		self.tvcolumn1.set_attributes( \
			__cell2, markup=Conf.TREEVIEW_FILE_TRANSFER_STATUS_PANEL_COLUMN_FILEPATH)
		self.tvcolumn2.set_attributes( \
			__cell7, markup=Conf.TREEVIEW_FILE_TRANSFER_STATUS_PANEL_COLUMN_FILE_SIZE)
		self.tvcolumn3.set_attributes( \
			__cell4, markup=Conf.TREEVIEW_FILE_TRANSFER_STATUS_PANEL_COLUMN_TIME_LEFT)
		self.tvcolumn4.set_attributes( \
			__cell5, markup=Conf.TREEVIEW_FILE_TRANSFER_STATUS_PANEL_COLUMN_PERCENT)
		self.tvcolumn5.set_attributes( \
			__cell6, markup=Conf.TREEVIEW_FILE_TRANSFER_STATUS_PANEL_COLUMN_SPEED)

		self.tvcolumn0.set_resizable(False)
		self.tvcolumn1.set_resizable(False)
		self.tvcolumn2.set_resizable(False)
		self.tvcolumn3.set_resizable(True)
		self.tvcolumn1.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
		self.tvcolumn3.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)

	def insert_row (self) :
		pass

	def addRows (self, result) :
		result = self.__reorderDirectoryFirst(result)
		for i in result :
			self.addRow(i)

	def addRow (self, **kwargs) :
		"""
		data is dict, which has the,
			key, value
			filepath
			filesize
		"""

		if kwargs["method"] == "put" :
			__direction = self.ICONS["up"]
		else :
			__direction = self.ICONS["down"]

		__uid = func.get_uid()
		__iter = self.WIDGET_TREESTORE.append( \
				[ \
					__direction, \
					self.ICONS["file"], \
					kwargs["filepath"], \
					__OS__.get_filesize_as_readable(int(kwargs["filesize"])), \
					"--:--", \
					"-", \
					"- Gbytes/s", \
					kwargs["filepath-local"], \
					__uid, \
				] \
		)

		kwargs["iter"] = __iter
		self.FILETRANSFERQUEUE.append(__uid, **kwargs)

		return __iter

	##################################################
	# EVENT METHOD

class TreeRemote (__TREEVIEW__.Core, WithGNOME) :

	FILE_TRANSFER = None
	PARENT = None

	HANDLER_ID_CURSOR_CHANGED = None

	def __init__ (self, xml, **kwargs) :
		PROPERTIES = { \
			"enable-search" : True, \
			"fixed-height-mode" : False, \
			"headers-clickable" : False, \
			"headers-visible" : False, \
			"hover-expand" : False, \
			"hover-selection" : False, \
			"reorderable" : False, \
			"rules-hint" : False, \
		}

		self.generateIcon()

		__TREEVIEW__.Core.__init__( \
			self, xml, "treeview-tree-remote", properties=PROPERTIES)

		self.HANDLER_ID_CURSOR_CHANGED = \
			self.WIDGET_TREEVIEW.connect("cursor-changed", self.on_cursor_changed)

	def getName (self, iter) :
		if not iter : return None

		return self.WIDGET_TREESTORE.get_value(iter, \
			Conf.TREEVIEW_TREE_PANEL_COLUMN_NAME)

	def create_frame (self) :
		# create a CellRenderers to render the data
		__cell1 = gtk.CellRendererPixbuf()
		__cell2 = gtk.CellRendererText()

		# set property.
		__cell1.set_property( \
			"width", self.ICONS["__default__"].get_property("width"))
		__cell1.set_property("xalign", 0.5)
		__cell2.set_property("xalign", 0.0)

		self.tvcolumn1 = gtk.TreeViewColumn("Name")
		self.tvcolumn1.set_expand(True)

		self.WIDGET_TREESTORE = gtk.ListStore( \
			gobject.TYPE_OBJECT, gobject.TYPE_STRING, \
			gobject.TYPE_STRING, \
		)

		self.WIDGET_TREEVIEW.set_model(self.WIDGET_TREESTORE)

		# add columns to treeview
		self.WIDGET_TREEVIEW.append_column(self.tvcolumn1)

		self.tvcolumn1.pack_start(__cell1, False)
		self.tvcolumn1.pack_start(__cell2, True)

		self.tvcolumn1.set_attributes( \
			__cell1, pixbuf=Conf.TREEVIEW_TREE_PANEL_COLUMN_ICON)
		self.tvcolumn1.set_attributes( \
			__cell2, text=Conf.TREEVIEW_TREE_PANEL_COLUMN_NAME)

		self.tvcolumn1.set_resizable(True)
		#self.tvcolumn2.set_resizable(True)
		#self.tvcolumn3.set_resizable(True)
		#self.tvcolumn1.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
		self.tvcolumn1.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)

	def insert_row (self) :
		pass

	def addRows (self, result) :
		result = self.__reorderDirectoryFirst(result)
		for i in result :
			self.addRow(i)

	def addRow (self, data) :

		__iter = self.WIDGET_TREESTORE.append( \
			[ \
				self.ICONS["directory"], \
				data[0], \
				str(int(data[2]["permissions"])), \
			] \
		)

		return __iter

	##################################################
	# EVENT METHOD
	def on_cursor_changed (self, widget) :
		__store, __iter = self.get_selected()

		__name = self.getName(__iter)

		self.FILE_TRANSFER.listFileRemote( \
			os.path.join( \
				os.path.normpath( \
					os.path.join(
						self.FILE_TRANSFER.CURRENT_DIRECTORY_REMOTE, "../"), \
					), \
				__name \
			)
		)

	##################################################
	# MISC METHOD
	def __reorderDirectoryFirst (self, result, removeDotDirectory=True) :

		__directory = dict()
		__file = dict()

		for i in result :
			if removeDotDirectory :
				if i[0] in [".", "..", ] :
					continue

			if not __OS__.is_directory_by_st_mode(i[2]["permissions"]) :
				continue

			__directory.update({ i[0] : i})

		__dk = __directory.keys()
		__dk.sort()

		__list = list()
		[__list.append(__directory[i]) for i in __dk]

		return __list

class FileListRemote \
		(WithGNOME, TreeViewFileList, __TREEVIEW__.Core) :

	FILE_TRANSFER = None
	PARENT = None

	def __init__ (self, xml, **kwargs) :
		PROPERTIES = { \
			"enable-search" : True, \
			"fixed-height-mode" : False, \
			"headers-clickable" : True, \
			"headers-visible" : True, \
			"hover-expand" : False, \
			"hover-selection" : False, \
			"reorderable" : True, \
			"rules-hint" : True, \
		}

		__TREEVIEW__.Core.__init__( \
			self, xml, "treeview-remote", properties=PROPERTIES)
		TreeViewFileList.__init__(self)

	def addRows (self, result) :
		result = self.__reorderDirectoryFirst(result)
		for i in result :
			self.addRow(i)

	def addRow (self, data) :

		__filetype = __OS__.get_filetype_by_st_mode(data[2]["permissions"])

		if __filetype == stat.S_IFDIR :
			__pixbuf = self.ICONS["directory"]
		elif __filetype == stat.S_IFLNK :
			__pixbuf = self.ICONS["link"]
		else :
			__pixbuf = self.ICONS["file"]

		__permission = __OS__.get_permission_oct2str( \
			__OS__.get_permission_st_mode2oct(data[2]["permissions"]))

		if __filetype != stat.S_IFREG :
			__size = ""
		else :
			__size = __OS__.get_filesize_as_readable(int(data[2]["size"]))

		__iter = self.WIDGET_TREESTORE.append( \
			[ \
				__pixbuf, \
				data[0], \
				__size, \
				__permission, \
				time.strftime("%D %R", time.localtime(data[2]["mtime"])), \
				str(int(data[2]["permissions"])), \
				os.path.join(self.FILE_TRANSFER.CURRENT_DIRECTORY_REMOTE, data[0]), \
			] \
		)

		return __iter

	##################################################
	# EVENT METHOD
	def on_refresh (self) :
		self.FILE_TRANSFER.listAllRemote()

	def on_remove (self, paths) :
		if len(paths) < 1 : return

		self.__on_remove(paths)

	def __on_remove (self, paths) :
		__filepath = \
			self.get_value(paths.pop(), \
			Conf.TREEVIEW_FILELIST_PANEL_COLUMN_FULL_PATH \
		)

		# find out whether it is file or directory.
		d = self.FILE_TRANSFER.FILE_TRANSFER.cmd_LS( \
			__OS__.get_parent_directory(__filepath) \
		)
		d.addCallback(self.__cb_on_remove, __filepath, paths)
		d.addErrback(self.FILE_TRANSFER.errBackRemote, "remove")

	def __cb_on_remove (self, result, filepath, paths) :
		__data = [ i for i in result if i[0] == os.path.basename(filepath) ][0]

		# if it is directory
		if __OS__.is_directory_by_st_mode( \
				__data[2]["permissions"], "find out real directory") :

			d = self.FILE_TRANSFER.FILE_TRANSFER.cmd_RMDIR(filepath)
			d.addCallback(self.__cb_on_remove0, paths)
			d.addErrback(self.FILE_TRANSFER.errBackRemote, "remove")

		# if it is file
		else :
			d = self.FILE_TRANSFER.FILE_TRANSFER.cmd_RM(filepath)
			d.addCallback(self.__cb_on_remove0, paths)
			d.addErrback(self.FILE_TRANSFER.errBackRemote, "remove")

	def __cb_on_remove0 (self, result, paths) :
		if len(paths) > 0 :
			self.__on_remove(paths)
		else :
			self.FILE_TRANSFER.listAllRemote(self.FILE_TRANSFER.CURRENT_DIRECTORY_REMOTE)

	def on_editing_canceled (self, cell) :
		cell.set_property("editable", False)

	def on_rename_edited (self, cell, path, new_text) :

		__name_old = self.get_value(path, Conf.TREEVIEW_FILELIST_PANEL_COLUMN_NAME)

		if __name_old == new_text :
			cell.set_property("editable", False)
			return

		__filepath = \
			self.get_value(path, Conf.TREEVIEW_FILELIST_PANEL_COLUMN_FULL_PATH)
		__new_path = os.path.join(self.FILE_TRANSFER.CURRENT_DIRECTORY_REMOTE, new_text)

		d = self.FILE_TRANSFER.FILE_TRANSFER.cmd_RENAME(__filepath, __new_path)
		d.addCallback(self.__cb_on_rename_edited, cell, path, __new_path)
		d.addErrback(self.__eb_on_rename_edited, path)

	def __cb_on_rename_edited (self, result, cell, path, new_path) :
		self.set_value( \
			path, Conf.TREEVIEW_FILELIST_PANEL_COLUMN_FULL_PATH, new_path)
		self.set_value( \
			path, Conf.TREEVIEW_FILELIST_PANEL_COLUMN_NAME, \
			os.path.basename(new_path) \
		)

		cell.set_property("editable", False)

		# reload file list.
		self.FILE_TRANSFER.listAllRemote(self.FILE_TRANSFER.CURRENT_DIRECTORY_REMOTE)

	def __eb_on_rename_edited (self, f, path) :
		self.FILE_TRANSFER.errBackRemote(failure.Failure())

		self.set_editable(self.get_iter(path))
		return

	def on_drag_data_get_data \
			(self, treeview, context, selection, target_id, etime) :
		for i in self.ROWS_CURRENT_SELECTED :
			__filepath = self.get_value(i, Conf.TREEVIEW_FILELIST_PANEL_COLUMN_FULL_PATH)

			prefix = os.path.dirname(__filepath)
			#dir_tmp = tempfile.mkdtemp( \
			#	dir=self.PARENT.STATE_GLOBAL.get_value("directory-temp") \
			#)
			dir_tmp = self.FILE_TRANSFER.CURRENT_DIRECTORY_LOCAL

			d = self.FILE_TRANSFER.FILE_TRANSFER.cmd_LS( \
				__OS__.get_parent_directory(__filepath))
			d.addCallback(self.__exportFiles, __filepath, prefix, dir_tmp)

		return True

	def __exportFiles (self, result, filepath, prefix, dir_tmp) :
		localpath = sre.compile("^/").sub("", \
			sre.compile("^%s" % sre.escape(prefix)).sub("", filepath))
		__data = [ i for i in result if i[0] == os.path.basename(filepath) ][0]

		__localpath = os.path.join(dir_tmp, __data[0])
		__remotepath = filepath
		if __OS__.is_directory_by_st_mode(__data[2]["permissions"]) :
			__OS__.mkdir(__localpath)
			self.FILE_TRANSFER.listAllLocal()
			self.__exportFilesDirectory(filepath, prefix, dir_tmp)
		else :
			self.FILE_TRANSFER.PANEL_FILETRANSFER_STATUS.addRow( \
				**{ \
					"filepath" : __remotepath, \
					"filesize" : __data[2]["size"], \
					"method" : "get", \
					"filepath-local" : __localpath, \
					"filepath-remote" : __remotepath, \
				} \
			)

	def __exportFilesDirectory (self, filepath, prefix, dir_tmp) :
		d = self.FILE_TRANSFER.FILE_TRANSFER.cmd_LS(filepath)
		d.addCallback(self.__exportFilesDirectory0, filepath, prefix, dir_tmp)

	def __exportFilesDirectory0 (self, result, filepath, prefix, dir_tmp) :
		localpath = sre.compile("^/").sub("", \
			sre.compile("^%s" % sre.escape(prefix)).sub("", filepath))

		for i in result :
			if i[0] in [".", "..", ] : continue
			__localpath = os.path.join(dir_tmp, localpath, i[0])
			__remotepath = os.path.join(filepath, i[0])
			if __OS__.is_directory_by_st_mode(i[2]["permissions"]) :
				__OS__.mkdir(__localpath)
				self.FILE_TRANSFER.listAllLocal()
				self.__exportFilesDirectory(__remotepath , prefix, dir_tmp)
			else :
				self.FILE_TRANSFER.PANEL_FILETRANSFER_STATUS.addRow( \
					**{ \
						"filepath" : __remotepath, \
						"filesize" : __data[2]["size"], \
						"method" : "get", \
						"filepath-local" : __localpath, \
						"filepath-remote" : __remotepath, \
					} \
				)

	def on_drag_data_received_data (self, treeview, context, x, y, selection, info, etime) :
		return

	def on_directory_up (self) :
		self.FILE_TRANSFER.on_directory_up_remote(None)

	def on_row_activated (self, treeview, path, view_column, user_param=None) :
		__iter = self.WIDGET_TREESTORE.get_iter(path)
		__name = self.getName(__iter)
		__perms = self.getPermissions(__iter)

		if __OS__.is_directory_by_st_mode(int(__perms)) : # if this is directory
			self.FILE_TRANSFER.listAllRemote(__name)
		else : # if this is not a directory, view file.
			return

	##################################################
	# MISC METHOD
	def __reorderDirectoryFirst (self, result, removeDotDirectory=True) :

		__directory = dict()
		__file = dict()

		for i in result :
			if removeDotDirectory :
				if i[0] in [".", "..", ] :
					continue

			if __OS__.is_directory_by_st_mode(i[2]["permissions"]) :
				__directory.update({ i[0] : i})
			else :
				__file.update({ i[0] : i})

		__dk = __directory.keys()
		__dk.sort()
		__fk = __file.keys()
		__fk.sort()

		__list = list()
		[__list.append(__directory[i]) for i in __dk]
		[__list.append(__file[i]) for i in __fk]

		return __list

class TreeLocal (__TREEVIEW__.Core, WithGNOME) :

	FILE_TRANSFER = None
	PARENT = None

	HANDLER_ID_CURSOR_CHANGED = None

	def __init__ (self, xml, **kwargs) :
		PROPERTIES = { \
			"enable-search" : True, \
			"fixed-height-mode" : False, \
			"headers-clickable" : False, \
			"headers-visible" : False, \
			"hover-expand" : False, \
			"hover-selection" : False, \
			"reorderable" : False, \
			"rules-hint" : False, \
		}

		self.generateIcon()

		__TREEVIEW__.Core.__init__( \
			self, xml, "treeview-tree-local", properties=PROPERTIES)

		self.HANDLER_ID_CURSOR_CHANGED = \
			self.WIDGET_TREEVIEW.connect("cursor-changed", self.on_cursor_changed)

	def getName (self, iter) :
		if not iter : return None

		return self.WIDGET_TREESTORE.get_value(iter, \
			Conf.TREEVIEW_TREE_PANEL_COLUMN_NAME)

	def create_frame (self) :
		# create a CellRenderers to render the data
		__cell1 = gtk.CellRendererPixbuf()
		__cell2 = gtk.CellRendererText()

		# set property.
		__cell1.set_property( \
			"width", self.ICONS["__default__"].get_property("width"))
		__cell1.set_property("xalign", 0.5)
		__cell2.set_property("xalign", 0.0)

		self.tvcolumn1 = gtk.TreeViewColumn("Name")
		self.tvcolumn1.set_expand(True)

		self.WIDGET_TREESTORE = gtk.ListStore( \
			gobject.TYPE_OBJECT, gobject.TYPE_STRING, \
			gobject.TYPE_STRING, \
		)

		self.WIDGET_TREEVIEW.set_model(self.WIDGET_TREESTORE)

		# add columns to treeview
		self.WIDGET_TREEVIEW.append_column(self.tvcolumn1)

		self.tvcolumn1.pack_start(__cell1, False)
		self.tvcolumn1.pack_start(__cell2, True)

		self.tvcolumn1.set_attributes( \
			__cell1, pixbuf=Conf.TREEVIEW_TREE_PANEL_COLUMN_ICON)
		self.tvcolumn1.set_attributes( \
			__cell2, text=Conf.TREEVIEW_TREE_PANEL_COLUMN_NAME)

		self.tvcolumn1.set_resizable(True)
		self.tvcolumn1.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)

	def insert_row (self) :
		pass

	def addRows (self, result) :
		result = self.__reorderDirectoryFirst(result)
		for i in result :
			self.addRow(i)

	def addRow (self, data) :

		__iter = self.WIDGET_TREESTORE.append( \
			[ \
				self.ICONS["directory"], \
				os.path.basename(data[0]), \
				str(int(data[1])), \
			] \
		)

		return __iter

	##################################################
	# EVENT METHOD
	def on_cursor_changed (self, widget) :
		__store, __iter = self.get_selected()

		__name = self.getName(__iter)

		self.FILE_TRANSFER.listFileLocal( \
			os.path.join( \
				os.path.normpath( \
					os.path.join(
						self.FILE_TRANSFER.CURRENT_DIRECTORY_LOCAL, "../"), \
					), \
				__name \
			)
		)

	##################################################
	# MISC METHOD
	def __reorderDirectoryFirst (self, result, removeDotDirectory=True) :

		__directory = dict()
		__file = dict()

		for i in result :
			if removeDotDirectory :
				if i[0] in [".", "..", ] :
					continue

			if not __OS__.is_directory_by_st_mode(i[1]) :
				continue

			__directory.update({ i[0] : i})

		__dk = __directory.keys()
		__dk.sort()

		__list = list()
		[__list.append(__directory[i]) for i in __dk]

		return __list

class FileListLocal \
		(WithGNOME, TreeViewFileList, __TREEVIEW__.Core) :

	def __init__ (self, xml, **kwargs) :
		PROPERTIES = { \
			"enable-search" : True, \
			"fixed-height-mode" : False, \
			"headers-clickable" : True, \
			"headers-visible" : True, \
			"hover-expand" : False, \
			"hover-selection" : False, \
			"reorderable" : False, \
			"rules-hint" : True, \
		}

		__TREEVIEW__.Core.__init__( \
			self, xml, "treeview-local", properties=PROPERTIES)
		TreeViewFileList.__init__(self)

	def addRows (self, result) :
		result = self.__reorderDirectoryFirst(result)
		for i in result :
			self.addRow(i)

	def addRow (self, data) :

		__filetype = __OS__.get_filetype_by_st_mode(data[1])

		if __filetype == stat.S_IFDIR :
			__pixbuf = self.ICONS["directory"]
		elif __filetype == stat.S_IFLNK :
			__pixbuf = self.ICONS["link"]
		else :
			__pixbuf = self.ICONS["file"]

		__permission = __OS__.get_permission_oct2str( \
			__OS__.get_permission_st_mode2oct(data[1]))

		if __filetype != stat.S_IFREG :
			__size = ""
		else :
			__size = __OS__.get_filesize_as_readable(int(data[7]))

		__iter = self.WIDGET_TREESTORE.append( \
			[ \
				__pixbuf, \
				os.path.basename(data[0]), \
				__size, \
				__permission, \
				time.strftime("%D %R", time.localtime(data[8])), \
				str(int(data[1])), \
				data[0], \
			] \
		)

		return __iter

	##################################################
	# EVENT METHOD
	def on_refresh (self) :
		self.FILE_TRANSFER.listAllLocal()

	def on_remove (self, paths) :
		for i in paths :
			__filepath = \
				self.get_value(i, Conf.TREEVIEW_FILELIST_PANEL_COLUMN_FULL_PATH)
			__retval = __OS__.remove(__filepath)

			if __retval != True :
				self.FILE_TRANSFER.errBackLocal(__retval, "remove")
				break

		self.FILE_TRANSFER.listAllLocal(self.FILE_TRANSFER.CURRENT_DIRECTORY_LOCAL)

	def on_editing_canceled (self, cell) :
		cell.set_property("editable", False)

	def on_rename_edited (self, cell, path, new_text) :

		__name_old = self.get_value(path, Conf.TREEVIEW_FILELIST_PANEL_COLUMN_NAME)
		if __name_old == new_text :
			cell.set_property("editable", False)
			return

		__filepath = \
			self.get_value(path, Conf.TREEVIEW_FILELIST_PANEL_COLUMN_FULL_PATH)
		__new_path = os.path.join(self.FILE_TRANSFER.CURRENT_DIRECTORY_LOCAL, new_text)

		try :
			os.rename(__filepath, __new_path)
		except OSError, message :
			func.format_log_string(self.FILE_TRANSFER.ENTRY_LOG_MESSAGE, _(message))

			self.set_editable(self.get_iter(path))
			return

		self.set_value( \
			path, Conf.TREEVIEW_FILELIST_PANEL_COLUMN_FULL_PATH, __new_path)
		self.set_value( \
			path, Conf.TREEVIEW_FILELIST_PANEL_COLUMN_NAME, new_text)

		cell.set_property("editable", False)

		# reload file list.
		self.FILE_TRANSFER.listAllLocal(self.FILE_TRANSFER.CURRENT_DIRECTORY_LOCAL)

	def on_drag_data_get_data (self, treeview, context, selection, target_id, etime) :
		dir_tmp = self.FILE_TRANSFER.CURRENT_DIRECTORY_REMOTE
		prefix = self.FILE_TRANSFER.CURRENT_DIRECTORY_LOCAL

		for i in self.ROWS_CURRENT_SELECTED :
			__filepath = self.get_value(i, \
				Conf.TREEVIEW_FILELIST_PANEL_COLUMN_FULL_PATH)

			self.__importDirectory0(__filepath, prefix, dir_tmp)

	def __importDirectory0 (self, localpath, prefix, dir_tmp) :

		__remotepath = os.path.join(dir_tmp, sre.compile("^%s" % sre.escape(prefix)).sub("", localpath))

		if __OS__.is_dir(localpath) :
			self.__importDirectory1(None, localpath, prefix, dir_tmp)

			d = self.FILE_TRANSFER.FILE_TRANSFER.cmd_MKDIR(__remotepath)
			self.__importDirectory1(None, localpath, prefix, dir_tmp)
			d.addErrback(self.FILE_TRANSFER.ignore)
			self.FILE_TRANSFER.listAllRemote()
		else :
			self.FILE_TRANSFER.PANEL_FILETRANSFER_STATUS.addRow( \
				**{ \
					"filepath" : localpath, \
					"filesize" : int(os.stat(localpath)[6]), \
					"method" : "put", \
					"filepath-local" : localpath, \
					"filepath-remote" : __remotepath, \
				} \
			)

		return True

	def __importDirectory1 (self, result, localpath, prefix, dir_tmp) :
		if not __OS__.is_dir(localpath) :
			return

		for i in os.listdir(localpath) :
			__full_path = os.path.join(localpath, i)
			self.__importDirectory0(__full_path, prefix, dir_tmp)

	def on_drag_data_received_data (self, treeview, context, x, y, selection, info, etime) :
		print "= received................."
		return

	def on_directory_up (self) :
		self.FILE_TRANSFER.on_directory_up_local(None)

	def on_row_activated (self, treeview, path, view_column, user_param=None) :
		__iter = self.WIDGET_TREESTORE.get_iter(path)
		__name = self.getName(__iter)
		__perms = self.getPermissions(__iter)

		if __OS__.is_directory_by_st_mode(int(__perms)) : # if this is directory
			self.FILE_TRANSFER.listAllLocal(__name)
			pass
		else : # if this is not a directory.
			__filepath = \
				self.get_value(path, Conf.TREEVIEW_FILELIST_PANEL_COLUMN_FULL_PATH)
			gnomevfs.url_show(gnomevfs.make_uri_canonical(__filepath))

	##################################################
	# MISC METHOD
	def __reorderDirectoryFirst (self, result, removeDotDirectory=True) :

		__directory = dict()
		__file = dict()

		for i in result :
			if removeDotDirectory :
				if i[0] in [".", "..", ] :
					continue

			if __OS__.is_directory_by_st_mode(int(i[1])) :
				__directory.update({ i[0] : i})
			else :
				__file.update({ i[0] : i})

		__dk = __directory.keys()
		__dk.sort()
		__fk = __file.keys()
		__fk.sort()

		__list = list()
		[__list.append(__directory[i]) for i in __dk]
		[__list.append(__file[i]) for i in __fk]

		return __list

class Bookmark (__TREEVIEW__.Core, WithGNOME) :

	STATE = None
	STATE_LOCAL = None
	STATE_GLOBAL = None
	GCONF_CLIENT = None

	BOOKMARK = None
	CALLBACK_FUNCTIONS = dict()
	NODELIST = dict()
	DOM_TMP = None

	def __init__ (self, xml, **kwargs) :
		"""
		kwargs is,
			state
		"""

		if not kwargs.has_key("callback_functions") :
			raise RuntimeError, _("callback_functions must be included.")
		else :
			self.CALLBACK_FUNCTIONS = kwargs["callback_functions"]

		if not kwargs.has_key("state") :
			raise RuntimeError, _("state must be included.")
		else :
			self.STATE = kwargs["state"]
			self.STATE_LOCAL = kwargs["state"]["local"]
			self.STATE_GLOBAL = kwargs["state"]["global"]
			self.GCONF_CLIENT = self.STATE_GLOBAL.get_value("gconf-client")
			self.BOOKMARK = self.STATE_GLOBAL.get_value("bookmark")

		PROPERTIES = { \
			"enable-search" : True, \
			"fixed-height-mode" : False, \
			"headers-clickable" : False, \
			"headers-visible" : False, \
			"hover-expand" : False, \
			"hover-selection" : False, \
			"reorderable" : True, \
			"rules-hint" : True, \
		}

		self.generateIcon()

		xml.get_widget("eventbox_bookmark").connect( \
			"event", self.on_eventbox_bookmark)

		__TREEVIEW__.Core.__init__( \
			self, xml, "treeview-bookmark", properties=PROPERTIES)

		#self.set_cursor((0,))

		# save bookmark periodically.
		#gobject.timeout_add(1000* 3, self.save_tree_to_xml)

	def save_tree_to_xml (self) :

		self.DOM_TMP = __XML__.create_new_document("them")

		self.WIDGET_TREESTORE.foreach(self.__save_tree_to_xml)

		fd = file(Conf.FILE_BOOKMARK, "w")
		fd.write(self.DOM_TMP.toprettyxml(indent='\t', newl='\n'))
		fd.close()

		return True

	def getUID (self, iter) :
		if not iter : return None

		return self.WIDGET_TREESTORE.get_value(iter, \
			Conf.TREEVIEW_BOOKMARK_PANEL_COLUMN_UID)

	def __save_tree_to_xml (self, model, path, iter, user_data=None) :
		__uid = self.getUID(iter)

		try :
			__node = self.BOOKMARK.getNodeByUID(__uid).cloneNode(False)
		except AttributeError :
			return

		self.NODELIST[path] = __node

		if len(path) == 1 :
			self.DOM_TMP.documentElement.appendChild(__node)
		else :
			__node_parent = self.NODELIST[path[:-1]]
			__node_parent.appendChild(__node)

	def create_frame (self) :

		# create a CellRenderers to render the data
		__cell1 = gtk.CellRendererPixbuf()
		__cell2 = gtk.CellRendererText()
		__cell3 = gtk.CellRendererText()

		# set property.
		__cell1.set_property("width", 50)
		__cell1.set_property("xalign", 0.5)
		__cell2.set_property("xalign", 0.0)

		__cell2.connect("edited", self.on_rename_edited)
		__cell2.connect("editing-canceled", self.on_editing_canceled)

		self.tvcolumn1 = gtk.TreeViewColumn()
		self.tvcolumn2 = gtk.TreeViewColumn()
		self.tvcolumn1.set_expand(True)

		self.WIDGET_TREESTORE = gtk.TreeStore( \
			gobject.TYPE_OBJECT, gobject.TYPE_STRING, gobject.TYPE_STRING)

		self.WIDGET_TREEVIEW.set_model(self.WIDGET_TREESTORE)

		# add columns to treeview
		self.WIDGET_TREEVIEW.append_column(self.tvcolumn1)

		self.tvcolumn1.pack_start(__cell1, False)
		self.tvcolumn1.pack_start(__cell2, False)
		self.tvcolumn2.pack_start(__cell3, False)

		self.tvcolumn1.set_attributes( \
			__cell1, pixbuf=Conf.TREEVIEW_BOOKMARK_PANEL_COLUMN_ICON)
		self.tvcolumn1.set_attributes( \
			__cell2, text=Conf.TREEVIEW_BOOKMARK_PANEL_COLUMN_NAME)
		self.tvcolumn2.set_attributes( \
			__cell3, text=Conf.TREEVIEW_BOOKMARK_PANEL_COLUMN_UID)

		self.tvcolumn1.set_resizable(True)
		self.tvcolumn1.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)

		#self.tvcolumn1.set_reorderable(True)

	def insert_row (self) :
		self.WIDGET_TREESTORE.clear()
		if not self.BOOKMARK : return

		for i in self.BOOKMARK.getChild() :
			self.add_row(i)

	def add_row (self, node, iter_parent=None) :

		data = self.BOOKMARK.getValues(node)

		# get icon
		if not self.ICONS.has_key(data["type"]) :
			__pixbuf = self.ICONS["__default__"]
		else :
			__pixbuf = self.ICONS[data["type"]]

		__iter = self.WIDGET_TREESTORE.append( \
			iter_parent, \
			[__pixbuf , data["label"], data["uid"], ] \
		)

		for i in self.BOOKMARK.getChild(node) :
			self.add_row(i, __iter)

		return __iter

	##################################################
	# EVENT HANDLER
	AVAILABLE_KEYNAMES_IN_PANEL_BOOKMARK = [ \
		"Right", \
		"Left", \
		"F2", \
		"Delete", \
		"<Control>Return", \
	]
	IS_EDITING = False
	def on_eventbox_bookmark (self, widget, event) :
		if self.IS_EDITING :
			return

		if event.type in [gtk.gdk.KEY_RELEASE, gtk.gdk.KEY_PRESS, ] :
			__keyname = func.get_keyname(event)

			if __keyname not in self.AVAILABLE_KEYNAMES_IN_PANEL_BOOKMARK :
				return

			(__treestore, __treeiter, ) = self.get_selected()
			__path = self.get_path(__treeiter)

			if __keyname == "Right" :
				self.WIDGET_TREEVIEW.expand_row(__path, False)

			elif __keyname == "Left" :
				"""
				If this iter has no child and so can not collapse, move the cursor
				to it's parent row.
				"""
				if self.WIDGET_TREESTORE.iter_has_child(__treeiter) :
					self.WIDGET_TREEVIEW.collapse_row(__path)
				else :
					# get parent
					__iter_parent = self.WIDGET_TREESTORE.iter_parent(__treeiter)
					if __iter_parent :
						self.set_cursor(self.get_path(__iter_parent))

			elif __keyname == "F2" and event.type not in [gtk.gdk.KEY_PRESS, ] :
				self.IS_EDITING = True
				(__store, __iter, ) = self.get_selected()
				self.set_editable( \
					__iter, 0, Conf.TREEVIEW_BOOKMARK_PANEL_COLUMN_NAME, \
				)

			elif __keyname == "Delete" and event.type not in [gtk.gdk.KEY_PRESS, ] :
				(__store, __iter, ) = self.get_selected()
				if not __iter : return

				__uid = self.getUID(__iter)

				# remove from xml
				__this_node = self.BOOKMARK.getNodeByUID(__uid)
				__parent_node = self.BOOKMARK.getParent(__this_node)

				if not __parent_node : return
				__parent_node.removeChild(__this_node)

				# remove from panel.
				self.remove(treeiter=__iter)

			elif __keyname == "<Control>Return" and event.type not in [gtk.gdk.KEY_PRESS, ] :
				__uid = self.getUID(__treeiter)
				if not self.BOOKMARK.getValuesByUID(__uid) : return

				# run the property dialog by it's TYPE.
				__data = self.BOOKMARK.getValuesByUID(__uid)

				__property_window = eval("Window.PROPERTY_%s" % __data["type"].upper())
				__property_window(__uid, panel=self, state=self.STATE)

			else :
				return

	###################################################################3
	# EVENT HANDLER
	def on_editing_canceled (self, cell) :
		cell.set_property("editable", False)
		self.IS_EDITING = False

	def on_rename_edited (self, cell, path, new_text) :
		cell.set_property("editable", False)

		__iter = self.WIDGET_TREESTORE.get_iter(path)
		__uid = self.getUID(__iter)
		__node = self.BOOKMARK.getNodeByUID(__uid)

		self.BOOKMARK.setValue(__node, "label", new_text)
		self.set_value(path, Conf.TREEVIEW_BOOKMARK_PANEL_COLUMN_NAME, new_text)

		self.IS_EDITING = False

	def on_row_activated (self, treeview, path, view_column, user_param=None) :
		__iter = self.WIDGET_TREESTORE.get_iter(path)
		__uid = self.getUID(__iter)

		__data = self.BOOKMARK.getValuesByUID(__uid)

		if __data["type"] == "category" :
			if self.WIDGET_TREEVIEW.row_expanded(path) :
				self.WIDGET_TREEVIEW.collapse_row(path)
			else :
				self.WIDGET_TREEVIEW.expand_row(path, False)

			return

		if not self.CALLBACK_FUNCTIONS.has_key("addPage") : return

		self.CALLBACK_FUNCTIONS["addPage"](__data["type"], **__data)

class ExternalCommand (__TREEVIEW__.Core) :

	STATE = None
	STATE_LOCAL = None
	STATE_GLOBAL = None
	NODELIST = dict()

	CALLBACK_FUNCTIONS = dict()

	def __init__ (self, xml, **kwargs) :
		"""
		kwargs is,
			state
		"""

		if not kwargs.has_key("state") :
			raise RuntimeError, _("state must be included.")
		else :
			self.STATE = kwargs["state"]
			self.STATE_LOCAL = kwargs["state"]["local"]
			self.STATE_GLOBAL = kwargs["state"]["global"]
			self.COMMAND = self.STATE_GLOBAL["command"]

		if not kwargs.has_key("callback_functions") :
			raise RuntimeError, _("callback_functions must be included.")
		else :
			self.CALLBACK_FUNCTIONS = kwargs["callback_functions"]

		PROPERTIES = { \
			"enable-search" : True, \
			"fixed-height-mode" : False, \
			"headers-clickable" : False, \
			"headers-visible" : False, \
			"hover-expand" : False, \
			"hover-selection" : False, \
			"reorderable" : True, \
			"rules-hint" : True, \
		}

		__TREEVIEW__.Core.__init__( \
			self, xml, "treeview-external-command", properties=PROPERTIES)

		self.WIDGET_TREEVIEW.connect("cursor-changed", self.on_select_command)

	def on_select_command (self, treeview) :
		(__store, __iter, ) = self.get_selected()
		__uid = self.getUID(__iter)

		if not __uid : return
		__data = self.COMMAND.getValuesByUID(__uid)

		if __data.has_key("value") and __data["value"].strip() :
			self.XML.get_widget("comboboxentry1").child.set_text(__data["value"])

	def save_tree_to_xml (self) :

		self.DOM_TMP = __XML__.create_new_document("command")

		self.WIDGET_TREESTORE.foreach(self.__save_tree_to_xml)

		fd = file(Conf.FILE_COMMAND, "w")
		fd.write(self.DOM_TMP.toprettyxml(indent='\t', newl='\n'))
		fd.close()

		return True

	def getUID (self, iter) :
		if not iter : return None

		return self.WIDGET_TREESTORE.get_value(iter, 1)

	def __save_tree_to_xml (self, model, path, iter, user_data=None) :
		__uid = self.getUID(iter)

		try :
			__node = self.COMMAND.getNodeByUID(__uid).cloneNode(False)
		except AttributeError :
			return

		self.NODELIST[path] = __node

		if len(path) == 1 :
			self.DOM_TMP.documentElement.appendChild(__node)
		else :
			__node_parent = self.NODELIST[path[:-1]]
			__node_parent.appendChild(__node)

	def create_frame (self) :

		# create a CellRenderers to render the data
		__cell0 = gtk.CellRendererText()
		__cell1 = gtk.CellRendererText()

		# set property.
		__cell0.set_property("xalign", 0.0)

		self.tvcolumn0 = gtk.TreeViewColumn()
		self.tvcolumn0.set_expand(True)

		self.WIDGET_TREESTORE = gtk.TreeStore( \
			gobject.TYPE_STRING, gobject.TYPE_STRING)

		self.WIDGET_TREEVIEW.set_model(self.WIDGET_TREESTORE)

		# add columns to treeview
		self.WIDGET_TREEVIEW.append_column(self.tvcolumn0)

		self.tvcolumn0.pack_start(__cell0, True)

		self.tvcolumn0.set_attributes(__cell0, text=0)
		self.tvcolumn0.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)

	def insert_row (self) :
		self.WIDGET_TREESTORE.clear()
		if not self.COMMAND : return

		for i in self.COMMAND.getChild() :
			self.add_row(i)

	def add_row (self, node) :
		data = self.COMMAND.getValues(node)
		self.WIDGET_TREESTORE.append(None, [data["value"], data["uid"], ])

	def on_row_activated (self, treeview, path, view_column, user_param=None) :

		__entry = self.XML.get_widget("comboboxentry1").child
		if not __entry.get_text().strip() : return

		self.CALLBACK_FUNCTIONS[ \
			"on_execute_external_command"]( \
			__entry \
		)

class ShortcutLocal (__TREEVIEW__.Core, WithGNOME) :

	STATE = None
	STATE_LOCAL = None
	STATE_GLOBAL = None
	NODELIST = dict()
	SHORTCUT = None
	FILEBROWSER = None

	def __init__ (self, xml, filebrowser, **kwargs) :
		"""
		kwargs is,
			state
		"""

		if not kwargs.has_key("state") :
			raise RuntimeError, _("state must be included.")
		else :
			self.STATE = kwargs["state"]
			self.STATE_LOCAL = kwargs["state"]["local"]
			self.STATE_GLOBAL = kwargs["state"]["global"]
			self.SHORTCUT = self.STATE_GLOBAL["shortcut-local"]

		PROPERTIES = { \
			"enable-search" : True, \
			"fixed-height-mode" : False, \
			"headers-clickable" : False, \
			"headers-visible" : False, \
			"hover-expand" : False, \
			"hover-selection" : False, \
			"reorderable" : False, \
			"rules-hint" : False, \
		}

		__TREEVIEW__.Core.__init__( \
			self, xml, "treeview-shortcut-local", properties=PROPERTIES)

		self.FILEBROWSER = filebrowser

		self.XML.get_widget("button34").connect("clicked", self.on_add_path)
		self.XML.get_widget("button35").connect("clicked", self.on_remove_path)

		self.WIDGET_TREEVIEW.connect( \
			"key-release-event", self.on_key_release_event)

	def save_tree_to_xml (self) :

		self.DOM_TMP = __XML__.create_new_document("paths")

		self.WIDGET_TREESTORE.foreach(self.__save_tree_to_xml)

		fd = file(Conf.FILE_SHORTCUT_LOCAL, "w")
		fd.write(self.DOM_TMP.toprettyxml(indent='\t', newl='\n'))
		fd.close()

		return True

	def getUID (self, iter) :
		if not iter : return None

		return self.WIDGET_TREESTORE.get_value( \
			iter, Conf.TREEVIEW_SHORTCUT_PANEL_COLUMN_UID)

	def __save_tree_to_xml (self, model, path, iter, user_data=None) :
		__uid = self.getUID(iter)

		try :
			__node = self.SHORTCUT.getNodeByUID(__uid).cloneNode(False)
		except AttributeError :
			return

		self.NODELIST[path] = __node

		if len(path) == 1 :
			self.DOM_TMP.documentElement.appendChild(__node)
		else :
			__node_parent = self.NODELIST[path[:-1]]
			__node_parent.appendChild(__node)

	def create_frame (self) :

		# create a CellRenderers to render the data
		__cell0 = gtk.CellRendererPixbuf()
		__cell1 = gtk.CellRendererText()
		__cell1.connect("edited", self.on_rename_edited)
		__cell1.connect("editing-canceled", self.on_editing_canceled)

		# set property.
		__cell0.set_property( \
			"width", self.ICONS["up"].get_property("width"))
		__cell1.set_property("xalign", 0.0)

		self.tvcolumn0 = gtk.TreeViewColumn()
		self.tvcolumn0.set_expand(True)

		self.WIDGET_TREESTORE = gtk.ListStore( \
			gobject.TYPE_OBJECT, gobject.TYPE_STRING, \
			gobject.TYPE_STRING, gobject.TYPE_STRING, \
		)

		self.WIDGET_TREEVIEW.set_model(self.WIDGET_TREESTORE)

		# add columns to treeview
		self.WIDGET_TREEVIEW.append_column(self.tvcolumn0)

		self.tvcolumn0.pack_start(__cell0, False)
		self.tvcolumn0.pack_start(__cell1, True)

		self.tvcolumn0.set_attributes( \
			__cell0, pixbuf=Conf.TREEVIEW_SHORTCUT_PANEL_COLUMN_ICON)
		self.tvcolumn0.set_attributes( \
			__cell1, text=Conf.TREEVIEW_SHORTCUT_PANEL_COLUMN_NAME)
		self.tvcolumn0.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)

	def insert_row (self) :
		self.WIDGET_TREESTORE.clear()
		if not self.SHORTCUT : return

		for i in self.SHORTCUT.getChild() :
			self.add_row(i)

	def add_row (self, node) :
		data = self.SHORTCUT.getValues(node)
		self.WIDGET_TREESTORE.append( \
			[self.ICONS["directory"], data["name"], data["value"], data["uid"], ] \
		)

	#####################################################################
	# EVENT HANDLER
	def on_add_path (self, widget) :
		__new_node = self.SHORTCUT.addNew()
		self.SHORTCUT.setValue( \
			__new_node, "value", self.FILEBROWSER.CURRENT_DIRECTORY_LOCAL)
		self.SHORTCUT.setValue( \
			__new_node, \
			"name", \
			os.path.basename( \
				os.path.normpath(self.FILEBROWSER.CURRENT_DIRECTORY_LOCAL) \
			) \
		)

		#self.SHORTCUT.getValue(__new_node, "uid")
		self.add_row(__new_node)

	def on_remove_path (self, widget) :
		__store, __iter = self.get_selected()

		if __iter is None : return

		__uid = self.getUID(__iter)
		if self.SHORTCUT.remove(__uid) :
			self.remove(treeiter=__iter)

	def on_row_activated (self, treeview, path, view_column, user_param=None) :
		self.FILEBROWSER.listAllLocal( \
			self.get_value(path, Conf.TREEVIEW_SHORTCUT_PANEL_COLUMN_VALUE) \
		)

	def set_editable (self, iter) :
		__path = self.get_path(iter)

		__column = self.WIDGET_TREEVIEW.get_column(0)
		cell = __column.get_cell_renderers()[1]
		cell.set_property("editable", True)

		self.WIDGET_TREEVIEW.set_cursor(__path, __column, True)

	def on_key_release_event (self, treeview, event) :
		__keyname = func.get_keyname(event)

		# remove
		if __keyname == "Delete" :
			self.on_remove_path(None)
			return True

		# rename
		elif __keyname == "F2" :
			__store, __iter = self.get_selected()
			if __iter is None : return

			self.set_editable(__iter)
			return True

		return False

	def on_editing_canceled (self, cell) :
		cell.set_property("editable", False)

	def on_rename_edited (self, cell, path, new_text) :
		cell.set_property("editable", False)

		__iter = self.get_iter(path)
		__uid = self.getUID(__iter)
		__node = self.SHORTCUT.getNodeByUID(__uid)

		self.SHORTCUT.setValue(__node, "name", new_text)
		self.set_value(path, Conf.TREEVIEW_SHORTCUT_PANEL_COLUMN_NAME, new_text)

class ShortcutRemote (__TREEVIEW__.Core, WithGNOME) :

	STATE = None
	STATE_LOCAL = None
	STATE_GLOBAL = None
	NODELIST = dict()
	SHORTCUT = None
	FILEBROWSER = None

	def __init__ (self, xml, filebrowser, **kwargs) :
		"""
		kwargs is,
			state
		"""

		self.FILEBROWSER = filebrowser

		if not kwargs.has_key("state") :
			raise RuntimeError, _("state must be included.")
		else :
			self.STATE = kwargs["state"]
			self.STATE_LOCAL = kwargs["state"]["local"]
			self.STATE_GLOBAL = kwargs["state"]["global"]
			self.SHORTCUT = self.FILEBROWSER.SHORTCUT_REMOTE

		PROPERTIES = { \
			"enable-search" : True, \
			"fixed-height-mode" : False, \
			"headers-clickable" : False, \
			"headers-visible" : False, \
			"hover-expand" : False, \
			"hover-selection" : False, \
			"reorderable" : False, \
			"rules-hint" : False, \
		}

		__TREEVIEW__.Core.__init__( \
			self, xml, "treeview-shortcut-remote", properties=PROPERTIES)

		self.XML.get_widget("button36").connect("clicked", self.on_add_path)
		self.XML.get_widget("button37").connect("clicked", self.on_remove_path)

	def save_tree_to_xml (self) :

		self.DOM_TMP = __XML__.create_new_document("paths")

		self.WIDGET_TREESTORE.foreach(self.__save_tree_to_xml)

		fd = file(self.FILEBROWSER.FILENAME_SHORTCUT_REMOTE, "w")
		fd.write(self.DOM_TMP.toprettyxml(indent='\t', newl='\n'))
		fd.close()

		return True

	def __save_tree_to_xml (self, model, path, iter, user_data=None) :
		__uid = self.getUID(iter)

		try :
			__node = self.SHORTCUT.getNodeByUID(__uid).cloneNode(False)
		except AttributeError :
			return

		self.NODELIST[path] = __node

		if len(path) == 1 :
			self.DOM_TMP.documentElement.appendChild(__node)
		else :
			__node_parent = self.NODELIST[path[:-1]]
			__node_parent.appendChild(__node)

	def getUID (self, iter) :
		if not iter : return None

		return self.WIDGET_TREESTORE.get_value(iter, Conf.TREEVIEW_SHORTCUT_PANEL_COLUMN_UID)

	def create_frame (self) :

		# create a CellRenderers to render the data
		__cell0 = gtk.CellRendererPixbuf()
		__cell1 = gtk.CellRendererText()
		__cell1.connect("edited", self.on_rename_edited)
		__cell1.connect("editing-canceled", self.on_editing_canceled)

		# set property.
		__cell0.set_property( \
			"width", self.ICONS["up"].get_property("width"))
		__cell1.set_property("xalign", 0.0)

		self.tvcolumn0 = gtk.TreeViewColumn()
		self.tvcolumn0.set_expand(True)

		self.WIDGET_TREESTORE = gtk.ListStore( \
			gobject.TYPE_OBJECT, gobject.TYPE_STRING, \
			gobject.TYPE_STRING, gobject.TYPE_STRING, \
		)

		self.WIDGET_TREEVIEW.set_model(self.WIDGET_TREESTORE)

		# add columns to treeview
		self.WIDGET_TREEVIEW.append_column(self.tvcolumn0)

		self.tvcolumn0.pack_start(__cell0, False)
		self.tvcolumn0.pack_start(__cell1, True)

		self.tvcolumn0.set_attributes( \
			__cell0, pixbuf=Conf.TREEVIEW_SHORTCUT_PANEL_COLUMN_ICON)
		self.tvcolumn0.set_attributes( \
			__cell1, text=Conf.TREEVIEW_SHORTCUT_PANEL_COLUMN_NAME)
		self.tvcolumn0.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)

	def insert_row (self) :
		self.WIDGET_TREESTORE.clear()
		if not self.SHORTCUT : return

		for i in self.SHORTCUT.getChild() :
			self.add_row(i)

	def add_row (self, node) :
		data = self.SHORTCUT.getValues(node)
		self.WIDGET_TREESTORE.append( \
			[self.ICONS["directory"], data["name"], data["value"], data["uid"], ] \
		)

	#####################################################################
	# EVENT HANDLER
	def on_add_path (self, widget) :
		__new_node = self.SHORTCUT.addNew()
		self.SHORTCUT.setValue( \
			__new_node, "value", self.FILEBROWSER.CURRENT_DIRECTORY_REMOTE)
		self.SHORTCUT.setValue( \
			__new_node, \
			"name", \
			os.path.basename( \
				os.path.normpath(self.FILEBROWSER.CURRENT_DIRECTORY_REMOTE) \
			) \
		)

		#self.SHORTCUT.getValue(__new_node, "uid")
		self.add_row(__new_node)
		self.save_tree_to_xml()

	def on_remove_path (self, widget) :
		__store, __iter = self.get_selected()

		if __iter is None : return

		__uid = self.getUID(__iter)
		if self.SHORTCUT.remove(__uid) :
			self.remove(treeiter=__iter)

		self.save_tree_to_xml()

	def on_row_activated (self, treeview, path, view_column, user_param=None) :
		self.FILEBROWSER.listAllRemote( \
			self.get_value(path, Conf.TREEVIEW_SHORTCUT_PANEL_COLUMN_VALUE) \
		)

	def set_editable (self, iter) :
		__path = self.get_path(iter)

		__column = self.WIDGET_TREEVIEW.get_column(0)
		cell = __column.get_cell_renderers()[1]
		cell.set_property("editable", True)

		self.WIDGET_TREEVIEW.set_cursor(__path, __column, True)

	def on_key_release_event (self, treeview, event) :
		__keyname = func.get_keyname(event)

		# remove
		if __keyname == "Delete" :
			self.on_remove_path(None)
			return True

		# rename
		elif __keyname == "F2" :
			__store, __iter = self.get_selected()
			if __iter is None : return

			self.set_editable(__iter)
			return True

		return False

	def on_editing_canceled (self, cell) :
		cell.set_property("editable", False)

	def on_rename_edited (self, cell, path, new_text) :
		cell.set_property("editable", False)

		__iter = self.get_iter(path)
		__uid = self.getUID(__iter)
		__node = self.SHORTCUT.getNodeByUID(__uid)

		self.SHORTCUT.setValue(__node, "name", new_text)
		self.set_value(path, Conf.TREEVIEW_SHORTCUT_PANEL_COLUMN_NAME, new_text)

class Preference (__TREEVIEW__.Core) :

	STATE = None
	STATE_LOCAL = None
	STATE_GLOBAL = None
	GCONF_CLIENT = None

	WIDGET_SIDE = None
	WINDOW = None

	def __init__ (self, xml, window, **kwargs) :
		"""
		kwargs is,
			state
		"""

		if not kwargs.has_key("state") :
			raise RuntimeError, _("state must be included.")
		else :
			self.STATE = kwargs["state"]
			self.STATE_LOCAL = kwargs["state"]["local"]
			self.STATE_GLOBAL = kwargs["state"]["global"]
			self.GCONF_CLIENT = self.STATE_GLOBAL.get_value("gconf-client")
			self.BOOKMARK = self.STATE_GLOBAL.get_value("bookmark")

		PROPERTIES = { \
			"enable-search" : False, \
			"fixed-height-mode" : False, \
			"headers-clickable" : False, \
			"headers-visible" : False, \
			"hover-expand" : False, \
			"hover-selection" : False, \
			"reorderable" : False, \
			"rules-hint" : False, \
		}

		self.WINDOW = window

		__TREEVIEW__.Core.__init__( \
			self, xml, "treeview-preference", properties=PROPERTIES)

		self.BOX_SIDE = self.XML.get_widget("box_preference_side")
		self.LABEL_SIDE = self.XML.get_widget("label_side")
		self.FRAME_SIDE = self.XML.get_widget("frame_side")
		self.FRAME_SIDE.hide()
		self.IMAGE_LOGO = self.XML.get_widget("image-preference")
		self.IMAGE_LOGO.set_from_file( \
			os.path.join(Conf__.DIR_PIXMAPS, "vee-logo.png"))

		self.HANDLER_ID_CURSOR_CHANGED = \
			self.WIDGET_TREEVIEW.connect("cursor-changed", self.on_cursor_changed)

	def create_frame (self) :

		# create a CellRenderers to render the data
		__cell0 = gtk.CellRendererText()

		# set property.
		__cell0.set_property("xalign", 0.0)

		self.tvcolumn0 = gtk.TreeViewColumn()
		self.tvcolumn0.set_expand(True)

		self.WIDGET_TREESTORE = gtk.TreeStore( \
			gobject.TYPE_STRING, gobject.TYPE_STRING)

		self.WIDGET_TREEVIEW.set_model(self.WIDGET_TREESTORE)

		# add columns to treeview
		self.WIDGET_TREEVIEW.append_column(self.tvcolumn0)

		self.tvcolumn0.pack_start(__cell0, False)

		self.tvcolumn0.set_attributes( \
			__cell0, text=Conf.TREEVIEW_PREFERENCE_PANEL_COLUMN_NAME)

		self.tvcolumn0.set_resizable(True)
		self.tvcolumn0.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)

		#self.tvcolumn1.set_reorderable(True)

	def insert_row (self) :
		self.WIDGET_TREESTORE.clear()

		for i in Conf.PREFERENCE_ITEM :
			if not USE_TWISTED :
				if i[0] == "SSH" : continue

			self.add_row(i, None)

	def add_row (self, data, iter_parent=None) :

		__iter = self.WIDGET_TREESTORE.append(iter_parent, [_(data[0]), data[1], ])

		if data[2] :
			for i in data[2] :
				self.add_row(i, __iter)

		return __iter

	##################################################
	# EVENT HANDLER
	def getUID (self, iter) :
		if not iter : return None

		return self.WIDGET_TREESTORE.get_value(iter, \
			Conf.TREEVIEW_PREFERENCE_PANEL_COLUMN_UID)

	XML_CURRNET = None
	def on_cursor_changed (self, widget) :
		__store, __iter = self.get_selected()
		__uid = self.getUID(__iter)

		if self.WIDGET_SIDE :
			self.WIDGET_SIDE.destroy()
		try :
			__func = getattr(self, "in_activate_%s" % __uid)
		except AttributeError :
			self.FRAME_SIDE.hide()
			self.IMAGE_LOGO.show()
		else :
			self.FRAME_SIDE.show()
			self.IMAGE_LOGO.hide()
			__title = self.get_value( \
				self.get_path(__iter), Conf.TREEVIEW_PREFERENCE_PANEL_COLUMN_NAME)

			self.LABEL_SIDE.set_markup("<span size=\"x-large\" weight=\"bold\">%s</span>" % _(__title))
			(self.XML_CURRNET, __widget, ) =self.__attach_widget(__uid)
			__func(__widget)

			self.__connect_handler(__uid)

	def __connect_handler (self, uid) :
		# connect signal
		try :
			__handler = getattr(Handler, "PREFERENCES_%s" % uid.upper())
		except AttributeError :
			return False

		__dict_handler = dict()
		for i, j in __handler.items() :
			__dict_handler[i] = eval("self.%s" % j)

		self.XML_CURRNET.signal_autoconnect(__dict_handler)

		return True

	def getWidgetCurrent (self, widget_name) :
		return self.XML_CURRNET.get_widget(widget_name)

	def __attach_widget (self, type) :
		__widget_name = "box_preference_%s" % type
		(__xml, __widget, ) = WIDGET.get_widget(__widget_name)
		WIDGET.translate(__xml, Conf.MAP_WIDGET[__widget_name]["message"])

		if not __widget :
			if self.WIDGET_SIDE :
				self.WIDGET_SIDE.destroy()

			self.FRAME_SIDE.hide()
			self.IMAGE_LOGO.show()
		else :
			self.BOX_SIDE.pack_start(__widget, True)
			self.WIDGET_SIDE = __widget

		return (__xml, __widget, )

	def in_activate_terminal (self, widget) :
		__terminal_command = self.GCONF_CLIENT.getValue("terminal_command")

		self.getWidgetCurrent("entry62").set_text(__terminal_command)
		#self.getWidgetCurrent("filechooserbutton7").set_filename(__shell)

	def in_activate_vim (self, widget) :
		__vim_command = self.GCONF_CLIENT.getValue("vim_command")

		self.getWidgetCurrent("entry63").set_text(__vim_command)

	def in_activate_external_command (self, widget) :
		self.getWidgetCurrent("entry65").set_text( \
			self.GCONF_CLIENT.getValue("external_command_shell_command"))
		self.getWidgetCurrent("checkbutton34").set_active( \
			self.GCONF_CLIENT.getValue("external_command_exit_without_enter") \
		)

	def in_activate_ssh (self, widget) :
		__combobox_ssh_version = self.getWidgetCurrent("combobox10")

		# create a CellRenderers to render the data
		__cell0 = gtk.CellRendererText()
		__cell0.set_property("xalign", 0.0)

		__treestore_ssh_version = gtk.TreeStore(str, str)
		__combobox_ssh_version.set_model(__treestore_ssh_version)

		# add columns to treeview
		__combobox_ssh_version.pack_start(__cell0, False)
		__combobox_ssh_version.set_attributes(__cell0, text=0)

		__treestore_ssh_version.append(None, [_("Version 2"), "2", ])
		__treestore_ssh_version.append(None, [_("Version 1"), "1", ])
		__default_ssh_version = self.GCONF_CLIENT.getValue("default_ssh_version")
		if __default_ssh_version == 1 :
			__combobox_ssh_version.set_active(1)
		else :
			__combobox_ssh_version.set_active(0)

		# set default ssh port
		self.getWidgetCurrent("entry64").set_text( \
			str(self.GCONF_CLIENT.getValue("default_ssh_port")))

	def in_activate_background (self, widget) :
		__radio_none					= self.getWidgetCurrent("radiobutton19")
		__radio_background_image	= self.getWidgetCurrent("radiobutton20")
		__radio_transparent			= self.getWidgetCurrent("radiobutton21")

		__vte_background_type = self.GCONF_CLIENT.getValue("vte_background_type")
		__vte_background_image_file = \
			self.GCONF_CLIENT.getValue("vte_background_image_file")
		__vte_background_shade = \
			self.GCONF_CLIENT.getValue("vte_background_shade")

		if __vte_background_type == "transparent" :
			__radio_transparent.set_active(True)

			self.getWidgetCurrent("hbox122").set_property("sensitive", False)
		elif __vte_background_type == "image-file" :
			__radio_background_image.set_active(True)
			self.getWidgetCurrent("vbox147").set_property("sensitive", True)

			if __vte_background_image_file :
				self.getWidgetCurrent("filechooserbutton18").set_filename( \
					__vte_background_image_file)
		else :
			__radio_none.set_active(True)

			self.getWidgetCurrent("hbox122").set_property("sensitive", False)
			self.getWidgetCurrent("vbox147").set_property("sensitive", False)

		__scale_shade = __vte_background_shade
		if __scale_shade < 0 or __scale_shade > 1 :
			__scale_shade = 0.2

		# set shade scale.
		self.getWidgetCurrent("hscale5").set_value(__scale_shade)

		__radio_none.connect( \
			"toggled", self.on_changed_vte_background_type, "none")
		__radio_background_image.connect( \
			"toggled", self.on_changed_vte_background_type, "image-file")
		__radio_transparent.connect( \
			"toggled", self.on_changed_vte_background_type, "transparent")

	def in_activate_font (self, widget) :
		# font #############################################
		self.getWidgetCurrent("checkbutton37").set_active( \
			self.GCONF_CLIENT.getValue("use_system_default_font") \
		)
		self.getWidgetCurrent("fontbutton1").set_font_name( \
			self.GCONF_CLIENT.getValue("font"))
		self.getWidgetCurrent("table79").set_property("sensitive", \
			not self.getWidgetCurrent("checkbutton37").get_active())

	def in_activate_color (self, widget) :
		__color_foreground = self.getWidgetCurrent("colorbutton21")
		__color_background = self.getWidgetCurrent("colorbutton22")
		__color_dim = self.getWidgetCurrent("colorbutton23")
		__color_highlight = self.getWidgetCurrent("colorbutton24")
		__color_cursor = self.getWidgetCurrent("colorbutton25")

		__color_foreground.set_color( \
			func.str_to_color( \
				self.GCONF_CLIENT.getValueByConf("color_foreground")))
		__color_background.set_color( \
			func.str_to_color( \
				self.GCONF_CLIENT.getValueByConf("color_background")))
		__color_dim.set_color( \
			func.str_to_color(self.GCONF_CLIENT.getValueByConf("color_dim")))
		__color_highlight.set_color( \
			func.str_to_color(self.GCONF_CLIENT.getValueByConf("color_highlight")))
		__color_cursor.set_color( \
			func.str_to_color(self.GCONF_CLIENT.getValueByConf("color_cursor")))

		__color_foreground.connect( \
			"color-set", self.on_changed_color, "color_foreground")
		__color_background.connect( \
			"color-set", self.on_changed_color, "color_background")
		__color_dim.connect("color-set", self.on_changed_color, "color_dim")
		__color_highlight.connect( \
			"color-set", self.on_changed_color, "color_highlight")
		__color_cursor.connect("color-set", self.on_changed_color, "color_cursor")

	def in_activate_etc (self, widget) :
		__enable_logging = self.GCONF_CLIENT.getValue("enable_logging")
		__log_directory = os.path.expanduser( \
			self.GCONF_CLIENT.getValue("log_directory") \
		)
		__enable_scrollbar = self.GCONF_CLIENT.getValue("enable_scrollbar")

		self.getWidgetCurrent("checkbutton36").set_active(__enable_logging)
		self.getWidgetCurrent("table78").set_property( \
			"sensitive", __enable_logging)
		if __log_directory :
			self.getWidgetCurrent("filechooserbutton19").set_current_folder( \
				__log_directory)

		self.getWidgetCurrent("spinbutton7").set_value( \
			int(self.GCONF_CLIENT.getValue("max_scroll_line")))

		self.getWidgetCurrent("checkbutton38").set_active(__enable_scrollbar)

	##################################################
	# EVENT HANDLER
	def on_terminal_changed_filechooser_command (self, chooser) :
		__new = chooser.get_filename()

		if not __new : return
		self.getWidgetCurrent("entry62").set_text(__new)

	def on_terminal_changed_command (self, entry) :
		__command = entry.get_text().strip()

		if len(__command) < 1 : return
		self.GCONF_CLIENT.setValue("terminal_command", __command)

	def on_vim_changed_command (self, entry) :
		__command = entry.get_text().strip()

		if len(__command) < 1 : return
		self.GCONF_CLIENT.setValue("vim_command", __command)

	def on_vim_changed_filechooser_command (self, chooser) :
		__new = chooser.get_filename()

		if not __new : return
		self.getWidgetCurrent("entry63").set_text(__new)

	def on_ssh_changed_ssh_version (self, combobox) :
		__treemodel = combobox.get_model()
		self.GCONF_CLIENT.setValue("default_ssh_version", \
			int(__treemodel.get_value(combobox.get_active_iter(), 1)))

	def on_ssh_changed_port (self, entry) :
		__new = entry.get_text().strip()

		if not __new : return

		try :
			int(__new)
		except ValueError :
			entry.set_text("22")
			return

		self.GCONF_CLIENT.setValue("default_ssh_port", int(__new))

	def on_external_command_changed_command (self, entry) :
		__command = entry.get_text().strip()

		if len(__command) < 1 : return
		self.GCONF_CLIENT.setValue("external_command_shell_command", __command)

	def on_external_command_changed_filechooser_command (self, chooser) :
		__new = chooser.get_filename()

		if not __new : return
		self.getWidgetCurrent("entry65").set_text(__new)

	def on_external_command_exit_without_enter (self, checkbutton) :
		self.GCONF_CLIENT.setValue( \
			"external_command_exit_without_enter", \
			checkbutton.get_active() \
		)

	def on_font_set (self, fontbutton) :
		self.GCONF_CLIENT.setValue("font", fontbutton.get_font_name())

	def on_use_system_font (self, checkbutton) :
		__active = checkbutton.get_active()
		self.GCONF_CLIENT.setValue( \
			"use_system_default_font", __active \
		)

		self.getWidgetCurrent("table79").set_property( \
			"sensitive", not __active \
		)
		try :
			self.getWidgetCurrent("table79").set_property( \
				"sensitive", not __active \
			)
		except AttributeError :
			pass

	def on_changed_vte_background_type (self, radiobutton, which) :
		if which == "transparent" :
			self.getWidgetCurrent("vbox147").set_property("sensitive", True)
			self.getWidgetCurrent("hbox122").set_property("sensitive", False)
		elif which == "image-file" :
			#self.getWidgetCurrent("vbox147").set_property("sensitive", False)
			self.getWidgetCurrent("hbox122").set_property("sensitive", True)
		else : # none
			self.getWidgetCurrent("hbox122").set_property("sensitive", False)
			self.getWidgetCurrent("vbox147").set_property("sensitive", False)

		self.GCONF_CLIENT.setValue("vte_background_type", which)

	def on_changed_background_image_file (self, chooser) :
		__new_filename = chooser.get_filename()
		if not __new_filename : return

		self.GCONF_CLIENT.setValue("vte_background_type", "image-file")
		self.GCONF_CLIENT.setValue("vte_background_image_file", __new_filename)

	def on_changed_saturation (self, hscale) :
		__value = round(hscale.get_value(), 1)
		self.GCONF_CLIENT.setValue("vte_background_shade", __value)

	def on_changed_max_scroll_line (self, spin) :
		__value = spin.get_value()

		self.GCONF_CLIENT.setValue("max_scroll_line", int(__value))

	def on_changed_color (self, button, which) :
		self.GCONF_CLIENT.setValue(which, func.color_to_str(button.get_color()))

	def on_changed_enable_scrollbar (self, checkbutton) :
		__enable = checkbutton.get_active()
		self.GCONF_CLIENT.setValue("enable_scrollbar", __enable)

	def on_changed_enable_logging (self, checkbutton) :
		__enable = checkbutton.get_active()
		try :
			self.getWidgetCurrent("table61").set_property("sensitive", __enable)
		except AttributeError :
			pass

		self.GCONF_CLIENT.setValue("enable_logging", __enable)

	def on_changed_directory_logging (self, chooser) :
		__directory = chooser.get_current_folder()
		if not __directory : return

		self.GCONF_CLIENT.setValue("log_directory", __directory)

class KeyboardShortcutForPreferences (__TREEVIEW__.Core) :

	def __init__ (self, xml, **kwargs) :
		"""
		kwargs is,
			state
		"""

		PROPERTIES = { \
			"enable-search" : False, \
			"fixed-height-mode" : False, \
			"headers-clickable" : False, \
			"headers-visible" : True, \
			"hover-expand" : False, \
			"hover-selection" : False, \
			"reorderable" : False, \
			"rules-hint" : False, \
		}

		self.STATE = kwargs["state"]
		self.STATE_LOCAL = kwargs["state"]["local"]
		self.STATE_GLOBAL = kwargs["state"]["global"]
		self.GCONF_CLIENT = self.STATE_GLOBAL.get_value("gconf-client")

		__TREEVIEW__.Core.__init__( \
			self, xml, "treeview-keyboard-shortcut", properties=PROPERTIES)

	def getUID (self, iter) :
		if not iter : return None

		return self.WIDGET_TREESTORE.get_value(iter, Conf.TREEVIEW_SHORTCUT_PANEL_COLUMN_UID)

	def create_frame (self) :

		# create a CellRenderers to render the data
		__cell0 = gtk.CellRendererText()
		__cell1 = gtk.CellRendererText()

		__tvcolumn0 = gtk.TreeViewColumn(_("Action"))
		__tvcolumn1 = gtk.TreeViewColumn(_("Shortcut"))

		__tvcolumn0.set_expand(True)
		__tvcolumn0.set_sort_column_id( \
			Conf.TREEVIEW_PREFERENCES_SHORTCUT_COLUMN_LABEL)

		self.WIDGET_TREESTORE = gtk.TreeStore(str, str, str)
		self.WIDGET_TREEVIEW.set_model(self.WIDGET_TREESTORE)

		self.WIDGET_TREEVIEW.append_column(__tvcolumn0)
		self.WIDGET_TREEVIEW.append_column(__tvcolumn1)

		__tvcolumn0.pack_start(__cell0, True)
		__tvcolumn1.pack_start(__cell1, True)

		__tvcolumn0.set_attributes( \
			__cell0, text=Conf.TREEVIEW_PREFERENCES_SHORTCUT_COLUMN_LABEL)
		__tvcolumn1.set_attributes( \
			__cell1, text=Conf.TREEVIEW_PREFERENCES_SHORTCUT_COLUMN_KEY)

		self.EVENTBOX = self.XML.get_widget("eventbox-keyboard-shortcut")
		self.EVENTBOX.set_events(gtk.gdk.KEY_RELEASE)
		self.EVENTBOX.add_events(gtk.gdk.KEY_PRESS)

		# connect event.
		self.WIDGET_TREEVIEW.connect( \
			"row-activated", self.on_accel_activated)

		self.EVENTBOX.connect("key-release-event", self.on_grab_keyval)

	def insert_row (self) :
		self.WIDGET_TREESTORE.clear()
		for __name, __child in Conf.KEYBOARD_SHORTCUT_LIST_FOR_TREEVIEW.items() :
			__iter = self.WIDGET_TREESTORE.append(None, [_(__name), "", "", ])

			for j in __child :
				self.add_row(__iter, j)

		self.WIDGET_TREEVIEW.expand_all()

	def add_row (self, parent, child) :
		__key_in_gconf = self.GCONF_CLIENT.getValue(child[0])
		if not __key_in_gconf :
			__key_in_gconf = _("Disabled")

		return self.WIDGET_TREESTORE.append( \
			parent, \
			[_(child[1]), __key_in_gconf, child[0], ] \
		)

	#####################################################################
	# EVENT HANDLER
	CURRENT_PATH = None
	def on_accel_activated \
			(self, treeview, path, view_column, user_param1=None) :
		self.SHORTCUT_KEY_VALUE_ORG = \
			self.get_value(path, Conf.TREEVIEW_PREFERENCES_SHORTCUT_COLUMN_KEY)

		if not self.SHORTCUT_KEY_VALUE_ORG.strip() : return

		self.WIDGET_TREESTORE[path][1] = _("Press Key...")
		self.CURRENT_PATH = path

		return

	def __accel_disable (self) :
		self.WIDGET_TREESTORE[self.CURRENT_PATH][1] = _("Disabled")

		__conf_key = self.get_value( \
				self.CURRENT_PATH, \
				Conf.TREEVIEW_PREFERENCES_SHORTCUT_COLUMN_UID \
		)
		self.__set_keyval(__conf_key, "")

		self.CURRENT_PATH = None

	def __accel_cancel (self) :
		self.WIDGET_TREESTORE[self.CURRENT_PATH][1] = self.SHORTCUT_KEY_VALUE_ORG
		self.CURRENT_PATH = None

	def get_value (self, path, column_number) :
		__tree_iter = self.WIDGET_TREESTORE.get_iter(path)
		__value = self.WIDGET_TREESTORE.get_value(__tree_iter, column_number)

		return __value

	def on_grab_keyval (self, widget, event) :
		if not self.CURRENT_PATH : return

		__conf_key = self.get_value( \
				self.CURRENT_PATH, \
				Conf.TREEVIEW_PREFERENCES_SHORTCUT_COLUMN_UID \
		)
		__keyval_name = gtk.gdk.keyval_name(event.keyval)

		#__mod = event.state & gtk.gdk.MODIFIER_MASK
		__mod = event.state & Conf.AVAILABLE_MODIFIER_MASK
		if __mod == Conf.NULL_MODIFIER_MASK :

			# if backspace, cancel the operation.
			if __keyval_name == "BackSpace" :
				self.__accel_disable()
			elif __keyval_name == "Escape" :
				self.__accel_cancel()
			elif __keyval_name != "Return" :
				self.__accel_cancel()

			return

		if not __keyval_name : return
		if not __keyval_name.strip() : return

		if not gtk.accelerator_valid(event.keyval, event.state) : return

		# check the symbols caused by SHIFT key.
		__key_string = gtk.gdk.keyval_name(event.keyval)
		if Conf.UPPER_SYMBOLS.has_key(__key_string) :
			mods = event.state & ~ gtk.gdk.SHIFT_MASK
		else :
			mods = event.state

		# store key event.
		if len(__key_string) == 1 : __key_string = __key_string.upper()

		__key_name = gtk.accelerator_name(event.keyval, mods)

		# filtering the close this window.
		if __key_name == "<Control>c" : self.__accel_cancel()

		__key_value = ">".join(__key_name.split(">")[:-1]) + ">" + \
			__key_string

		# check this key is already in use.
		self.__KEYBOARD_SHORTCUT_IN_USE = None
		self.WIDGET_TREESTORE.foreach(self.__search_keyval_in_column, __key_value)

		if self.__KEYBOARD_SHORTCUT_IN_USE and \
				len(self.__KEYBOARD_SHORTCUT_IN_USE) == 2:
			Dialog.ShortcutKeyInUse(__key_value, self.__KEYBOARD_SHORTCUT_IN_USE[1])
			self.__accel_cancel()
			return

		# update gconf value.
		if not self.__set_keyval(__conf_key, __key_value) :
			raise RuntimeError, \
				_("Keyboard shortcut did not update, caused by gconf.")

		self.WIDGET_TREESTORE[self.CURRENT_PATH][1] = __key_value
		self.CURRENT_PATH = None

		return

	def __set_keyval (self, conf_key, keyval) :
		if not self.GCONF_CLIENT.setValue(conf_key, keyval) :
			return False
		else :
			return True

	def __search_keyval_in_column (self, model, path, iter, key_value) :
		if self.get_value( \
				path, Conf.TREEVIEW_PREFERENCES_SHORTCUT_COLUMN_KEY) == key_value :
			self.__KEYBOARD_SHORTCUT_IN_USE = (path, self.get_value( \
					path, Conf.TREEVIEW_PREFERENCES_SHORTCUT_COLUMN_LABEL), \
			)

			return True
		else :
			return False

"""
Description
-----------


ChangeLog
---------


Usage
-----


"""

__author__ =  "Spike^ekipS <spikeekips@gmail.com>"
__version__=  "0.1"
__nonsense__ = ""

__file__ = "Panel.py"


