# -*- coding: utf-8 -*-
#    Copyright 2005 Spike^ekipS <spikeekips@gmail.com>
#
#       This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

ITEM_DATA_SHORTCUT = { \
	"value" : "str", \
	"name" : "str", \
	"uid" : "str", \
}

ITEM_DATA_COMMAND = { \
	"value" : "str", \
	"uid" : "str", \
}

ITEM_DATA_BOOKMARK = { \
	"color_foreground": "str", \
	"uid": "str", \
	"external_command_exit_without_enter": "bool", \
	"color_cursor": "str", \
	"color_dim": "str", \
	"color_highlight": "str", \
	"label": "unicode", \
	"font": "str", \
	"vte_background_shade": "float", \
	"filename": "str", \
	"external_command": "str", \
	"type": "str", \
	"max_scroll_line": "int", \
	"vte_background_type": "str", \
	"host": "str", \
	"color_background": "str", \
	"user": "str", \
	"path": "str", \
	"timestamp_added": "float", \
	"use_system_default_font": "bool", \
	"vte_background_image_file": "str", \
	"command": "str", \
	"password" : "str", \
	"port" : "int", \
	"enable_logging" : "bool", \
	"file_logging" : "str", \
	"enable_scrollbar" : "bool", \
}

TEMPLATE_ONE = { \
	"__default__" : { \
		"uid" : "str", \
		"timestamp_added" : "float", \
		"label" : "unicode", \
		"type" : "str", \
	}, \
	"__vte__" : { \
		"use_system_default_font" : "bool", \
		"font" : "str", \
		"color_foreground" : "str", \
		"color_background" : "str", \
		"color_dim" : "str", \
		"color_highlight" : "str", \
		"color_cursor" : "str", \
		"max_scroll_line" : "int", \
		"vte_background_shade" : "float", \
		"vte_background_type" : "str", \
		"vte_background_image_file" : "str", \
		"command" : "str", \
		"enable_logging" : "bool", \
		"file_logging" : "str", \
		"enable_scrollbar" : "bool", \
	}, \
	"category" : { \
	}, \
	"vim" : { \
		"filename" : "str", \
	}, \
	"gvim" : { \
		"filename" : "str", \
	}, \
	"terminal" : { \
		"path" : "str", \
	}, \
	"external_command" : { \
		"external_command" : "str", \
		"external_command_exit_without_enter" : "bool" \
	}, \
	"ssh" : { \
		"host" : "str", \
		"user" : "str", \
		"port" : "int", \
		"password" : "str", \
		"external_command" : "str", \
	}, \
	"ssh_filebrowser" : { \
		"host" : "str", \
		"user" : "str", \
		"port" : "int", \
		"password" : "str", \
		"external_command" : "str", \
	}, \
}

TEMPLATE_DEPENDENCY = { \
	"vim"			: ("__vte__", "__default__") ,\
	"gvim"		: ("__vte__", "__default__") ,\
	"terminal"	: ("__vte__", "__default__") ,\
	"ssh"			: ("__vte__", "__default__") ,\
	"ssh_filebrowser"		: ("__vte__", "__default__") ,\
}

import xml
import xml.parsers.expat as Expat
from xml.dom import minidom
import func, __XML__

class Bookmark (__XML__.__XML_PARSER__) :

	def __init__ (self, xml_filename=None) :
		if not xml_filename :
			self.DOM = minidom.parseString("<?xml version=\"1.0\"?><them></them>")
		else :
			try :
				__XML__.__XML_PARSER__.__init__(self, xml_filename)
			except Expat.ExpatError :
				self.DOM = minidom.parseString("<?xml version=\"1.0\"?><them></them>")

	def __makeXPath (self, path, uid=None) :
		"""
		Read gtk treestore style 'path' and make xpath.

		@param :
			path : tuple(), ex) (0, 1,)

		@return :
			xpath : str()
		"""
		if type(path) is not tuple : return None

		if not uid :
			__xpath = "/them/%s" % "/".join(["one", ] * len(path))
		else :
			__xpath = "/them/%s[@uid = '%s']" % ("/".join(["one", ] * len(path)), uid)

		return __xpath

	def getOnes (self, path, uid=None) :

		xpath = self.__makeXPath(path, uid)

		if not xpath : return list()

		nodes = __XML__.evaluate_path(self.DOM, xpath)

		return nodes

	def getNodeByUID (self, uid) :
		nodes = __XML__.evaluate_path(self.DOM, "//one[ @uid = '%s' ]" % uid)

		if not nodes : return None

		return nodes[0]

	def getValuesByUID (self, uid) :
		return self.getValues(self.getNodeByUID(uid))

	def getValues (self, node) :
		if not hasattr(node, "attributes") : return None

		return __XML__.get_attributes(node.attributes, ITEM_DATA_BOOKMARK)

	def getParent (self, node) :
		return node.parentNode

	def getChild (self, node_parent=None) :
		"""
		If node_parent is None, start top root.
		"""
		if not node_parent :
			__child = list()
			for i in self.getOnes((0,)) :
				if i.nodeType == i.TEXT_NODE : continue
				__child.append(i)
		else :
			__child = list()
			for i in node_parent.childNodes :
				if i.nodeType == i.TEXT_NODE : continue
				__child.append(i)

		return __child

	def createNewOne (self, type) :
		__template = TEMPLATE_ONE[type]

		if TEMPLATE_DEPENDENCY.has_key(type) :
			for i in TEMPLATE_DEPENDENCY[type] :
				__template.update(TEMPLATE_ONE[i])

		new = self.DOM.createElement("one")

		for i in __template :
			new.setAttribute(i, str())

		new.setAttribute("uid", func.get_uid())
		new.setAttribute("type", type)

		return new

	def getValue (self, node, key) :
		if not hasattr(node, "attributes") : return None
		if not node.attributes.has_key(key) : return None

		return __XML__.casting( \
			ITEM_DATA_BOOKMARK[key], node.attributes.get(key).nodeValue)

	def setValue (self, node, key, value) :
		if not hasattr(node, "attributes") : return False
		if not ITEM_DATA_BOOKMARK.has_key(key) : return False

		node.setAttribute(key, str(value))

		return True

	def updateValues (self, node, values=dict()) :
		if not hasattr(node, "attributes") : return False

		for __key, __value in values.items() :
			if not ITEM_DATA_BOOKMARK.has_key(__key) : continue
			node.setAttribute(__key, str(__value))

		return True

	def addNew (self, type, node_parent=None) :
		if not TEMPLATE_ONE.has_key(type) : return False

		# if node_parent is None, parent node is "/them"
		if not node_parent :
			__nodes = __XML__.evaluate_path(self.DOM, "/them")
			if not __nodes : return False
			if len(__nodes) < 1 : return False

			node_parent = __nodes[0]

		# create new node.
		__node_new = self.createNewOne(type)

		# add node to parent node
		return node_parent.appendChild(__node_new)


class Command (__XML__.__XML_PARSER__) :

	def __init__ (self, xml_filename=None) :
		if xml_filename :
			try :
				__XML__.__XML_PARSER__.__init__(self, xml_filename)
			except Expat.ExpatError :
				self.DOM = minidom.parseString("<?xml version=\"1.0\"?><commands></commands>")
		else :
			self.DOM = minidom.parseString("<?xml version=\"1.0\"?><commands></commands>")

	def __makeXPath (self, path, uid=None) :
		"""
		Read gtk treestore style 'path' and make xpath.

		@param :
			path : tuple(), ex) (0, 1,)

		@return :
			xpath : str()
		"""
		if type(path) is not tuple : return None

		if not uid :
			__xpath = "/commands/%s" % "/".join(["command", ] * len(path))
		else :
			__xpath = "/commands/%s[@uid = '%s']" % ("/".join(["command", ] * len(path)), uid)

		return __xpath

	def getOnes (self, path, uid=None) :

		xpath = self.__makeXPath(path, uid)

		if not xpath : return list()

		nodes = __XML__.evaluate_path(self.DOM, xpath)

		return nodes

	def getNodeByUID (self, uid) :
		nodes = __XML__.evaluate_path(self.DOM, "//command[ @uid = '%s' ]" % uid)

		if not nodes : return None

		return nodes[0]

	def getValuesByUID (self, uid) :
		return self.getValues(self.getNodeByUID(uid))

	def getValues (self, node) :
		if not hasattr(node, "attributes") : return None

		return __XML__.get_attributes(node.attributes, ITEM_DATA_BOOKMARK)

	def getParent (self, node) :
		return node.parentNode

	def getChild (self, node_parent=None) :
		"""
		If node_parent is None, start top root.
		"""
		if not node_parent :
			__child = list()
			for i in self.getOnes((0,)) :
				if i.nodeType == i.TEXT_NODE : continue
				__child.append(i)
		else :
			__child = list()
			for i in node_parent.childNodes :
				if i.nodeType == i.TEXT_NODE : continue
				__child.append(i)

		return __child

	def createNewOne (self) :
		new = self.DOM.createElement("command")
		new.setAttribute("uid", func.get_uid())

		return new

	def getValue (self, node, key) :
		if not hasattr(node, "attributes") : return None
		if not node.attributes.has_key(key) : return None

		return __XML__.casting( \
			ITEM_DATA_COMMAND[key], node.attributes.get(key).nodeValue)

	def setValue (self, node, key, value) :
		if not hasattr(node, "attributes") : return False
		if not ITEM_DATA_COMMAND.has_key(key) : return False

		node.setAttribute(key, str(value))

		return True

	def updateValues (self, node, values=dict()) :
		if not hasattr(node, "attributes") : return False

		for __key, __value in values.items() :
			if not ITEM_DATA_COMMAND.has_key(__key) : continue
			node.setAttribute(__key, str(__value))

		return True

	def addNew (self) :

		__nodes = __XML__.evaluate_path(self.DOM, "/commands")
		if not __nodes : return False
		if len(__nodes) < 1 : return False

		node_parent = __nodes[0]

		# create new node.
		__node_new = self.createNewOne()

		# add node to parent node
		return node_parent.appendChild(__node_new)

class Shortcut (__XML__.__XML_PARSER__) :

	def __init__ (self, xml_filename=None) :
		if xml_filename :
			try :
				__XML__.__XML_PARSER__.__init__(self, xml_filename)
			except Expat.ExpatError :
				self.DOM = minidom.parseString("<?xml version=\"1.0\"?><paths></paths>")
			except IOError :
				self.DOM = minidom.parseString("<?xml version=\"1.0\"?><paths></paths>")
		else :
			self.DOM = minidom.parseString("<?xml version=\"1.0\"?><paths></paths>")

	def __makeXPath (self, path, uid=None) :
		"""
		Read gtk treestore style 'path' and make xpath.

		@param :
			path : tuple(), ex) (0, 1,)

		@return :
			xpath : str()
		"""
		if type(path) is not tuple : return None

		if not uid :
			__xpath = "/paths/%s" % "/".join(["path", ] * len(path))
		else :
			__xpath = "/paths/%s[@uid = '%s']" % ("/".join(["path", ] * len(path)), uid)

		return __xpath

	def getOnes (self, path, uid=None) :

		xpath = self.__makeXPath(path, uid)

		if not xpath : return list()

		nodes = __XML__.evaluate_path(self.DOM, xpath)

		return nodes

	def getNodeByUID (self, uid) :
		nodes = __XML__.evaluate_path(self.DOM, "//path[ @uid = '%s' ]" % uid)

		if not nodes : return None

		return nodes[0]

	def getValuesByUID (self, uid) :
		return self.getValues(self.getNodeByUID(uid))

	def getValues (self, node) :
		if not hasattr(node, "attributes") : return None

		return __XML__.get_attributes(node.attributes, ITEM_DATA_BOOKMARK)

	def getParent (self, node) :
		return node.parentNode

	def getChild (self, node_parent=None) :
		"""
		If node_parent is None, start top root.
		"""
		if not node_parent :
			__child = list()
			for i in self.getOnes((0,)) :
				if i.nodeType == i.TEXT_NODE : continue
				__child.append(i)
		else :
			__child = list()
			for i in node_parent.childNodes :
				if i.nodeType == i.TEXT_NODE : continue
				__child.append(i)

		return __child

	def createNewOne (self) :
		new = self.DOM.createElement("path")
		new.setAttribute("uid", func.get_uid())

		return new

	def getValue (self, node, key) :
		if not hasattr(node, "attributes") : return None
		if not node.attributes.has_key(key) : return None

		return __XML__.casting( \
			ITEM_DATA_SHORTCUT[key], node.attributes.get(key).nodeValue)

	def setValue (self, node, key, value) :
		if not hasattr(node, "attributes") : return False
		if not ITEM_DATA_SHORTCUT.has_key(key) : return False

		node.setAttribute(key, str(value))

		return True

	def updateValues (self, node, values=dict()) :
		if not hasattr(node, "attributes") : return False

		for __key, __value in values.items() :
			if not ITEM_DATA_SHORTCUT.has_key(__key) : continue
			node.setAttribute(__key, str(__value))

		return True

	def addNew (self) :

		__nodes = __XML__.evaluate_path(self.DOM, "/paths")
		if not __nodes :
			print "no nodes."
			return False
		if len(__nodes) < 1 :
			return False

		node_parent = __nodes[0]

		# create new node.
		__node_new = self.createNewOne()

		# add node to parent node
		return node_parent.appendChild(__node_new)

	def remove (self, uid) :
		__this_node = self.getNodeByUID(uid)
		__parent_node = self.getParent(__this_node)

		if not __parent_node : return False
		__parent_node.removeChild(__this_node)

		return True


"""
Description
-----------


ChangeLog
---------


Usage
-----


"""

__author__ =  "Spike^ekipS <spikeekips@gmail.com>"
__version__=  "0.1"
__nonsense__ = ""

__file__ = "XMLHandler.py"


