# -*- coding: utf-8 -*-
#    Copyright 2005 Spike^ekipS <spikeekips@gmail.com>
#
#       This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import os
import Conf, Handler, __VEE__, WIDGET, func, GNOME

class CORE (__VEE__.Core, WIDGET.WIDGET) :

	NOTEBOOK = None
	PANEL_BOOKMARK = None
	TREEITER = None
	UID = None
	NODE = None
	DATA = None

	def __init__ (self, type, uid, **kwargs) :

		__VEE__.Core.__init__(self, **kwargs)
		self.BOOKMARK = self.STATE_GLOBAL["bookmark"]

		self.UID = uid

		if not kwargs.has_key("panel") :
			raise RunetimeError, "panel must be given."
		else :
			self.PANEL_BOOKMARK = kwargs["panel"]
			(__treestore, self.TREEITER, ) = self.PANEL_BOOKMARK.get_selected()

		self.NODE = self.BOOKMARK.getNodeByUID(self.UID)

		# get data by uid
		self.DATA = self.BOOKMARK.getValuesByUID(self.UID)

		WIDGET.WIDGET.__init__(self, "bookmark_property_%s" % type)
		self.NOTEBOOK = self.getWidget("notebook")

	def initialize (self) :

		# GENERAL ########################################################
		# set type image
		__icon = self.__getIconPixbuf(Conf.ICONS[self.DATA["type"]])
		self.getWidget("image1").set_from_pixbuf(__icon)

		# set label
		self.getWidget("entry1").set_text(self.DATA["label"])

	def __getIconPixbuf (self, icons=list()) :
		__gnome_icon = GNOME.Icon()
		__gnome_icon.SIZE = 96

		__icon = None
		for i in icons :
			__icon = __gnome_icon.getPixbufByIcon(i)
			if __icon : break

		if not __icon :
			__icon = __gnome_icon.getNone()

		return __icon

	def on_changed_label (self, entry) :
		__new_label = entry.get_text().strip()

		if len(__new_label) < 1 : return

		self.BOOKMARK.setValue(self.NODE, "label", __new_label)

		if self.TREEITER :
			self.PANEL_BOOKMARK.set_value( \
				self.PANEL_BOOKMARK.get_path(self.TREEITER), \
				Conf.TREEVIEW_BOOKMARK_PANEL_COLUMN_NAME, \
				__new_label \
			)

class VTE (CORE) :

	XML_FONT = None
	XML_COLOR = None
	XML_BACKGROUND = None
	XML_ETC = None

	def initialize (self) :
		CORE.initialize (self)

		# font ########################################################
		(self.XML_FONT, __widget, ) = WIDGET.get_widget("bookmark_property_frame_font")
		WIDGET.translate( \
			self.XML_FONT, Conf.MAP_WIDGET["bookmark_property_frame_font"]["message"])
		self.getWidget("box_font").pack_start(__widget, True)

		# color #######################################################
		(self.XML_COLOR, __widget, ) = WIDGET.get_widget("bookmark_property_frame_color")
		WIDGET.translate( \
			self.XML_COLOR, Conf.MAP_WIDGET["bookmark_property_frame_color"]["message"])
		self.getWidget("box_color").pack_start(__widget, True)

		# background ##################################################
		(self.XML_BACKGROUND, __widget, ) = WIDGET.get_widget("bookmark_property_frame_background")
		WIDGET.translate( \
			self.XML_BACKGROUND, \
			Conf.MAP_WIDGET["bookmark_property_frame_background"]["message"] \
		)
		self.getWidget("box_background").pack_start(__widget, True)

		# etc #########################################################
		(self.XML_ETC, __widget, ) = WIDGET.get_widget("bookmark_property_frame_etc")
		WIDGET.translate( \
			self.XML_ETC, Conf.MAP_WIDGET["bookmark_property_frame_etc"]["message"])
		self.getWidget("box_etc").pack_start(__widget, True)

		for i in [self.XML_FONT, self.XML_COLOR, self.XML_BACKGROUND, self.XML_ETC, ] :
			__dict_handler = dict()
			for k, m in Handler.__PROPERTY_BOOKMARK_VTE__.items() :
				__dict_handler[k] = eval("self.%s" % m)

			i.signal_autoconnect(__dict_handler)

		self.__initialize_vte()

		# END ############################################################
		self.WIDGET_SELF.set_property("resizable", False)
		self.getWidget("entry1").grab_focus()

	##################################################
	# EVENT HANDLER
	def on_changed_enable_scrollbar (self, checkbutton) :
		__enable = checkbutton.get_active()
		self.BOOKMARK.setValue( \
			self.NODE, \
			"enable_scrollbar",
			__enable \
		)

	def on_use_system_font (self, checkbutton) :
		__buttonFont = self.XML_FONT.get_widget("fontbutton1")
		__labelFont = self.XML_FONT.get_widget("hbox3")

		if checkbutton.get_active() :
			__buttonFont.set_sensitive(False)
			__labelFont.set_sensitive(False)
		else :
			__buttonFont.set_sensitive(True)
			__labelFont.set_sensitive(True)

		self.BOOKMARK.setValue( \
			self.NODE, \
			"use_system_default_font",
			checkbutton.get_active() \
		)

	def on_font_set (self, fontbutton) :
		self.BOOKMARK.setValue(self.NODE, "font", fontbutton.get_font_name())

	def on_changed_enable_logging (self, checkbutton) :
		__enable = checkbutton.get_active()
		try :
			self.XML_ETC.get_widget("table3").set_property("sensitive", __enable)
		except AttributeError :
			pass

		self.BOOKMARK.setValue(self.NODE, "enable_logging", __enable)

	def on_file_dialog_file_logging (self, button) :
		__current_path = self.XML_ETC.get_widget("entry2").get_text()

		if not os.path.exists(os.path.dirname(__current_path)) :
			__current_path = os.getenv("HOME")

		FileDialog_FileLogging( \
			state=self.STATE, \
			filepath=__current_path, \
			entry_filename=self.XML_ETC.get_widget("entry2") \
		)

	def on_changed_file_logging (self, entry) :
		__new_filename = entry.get_text()
		if not __new_filename : return

		self.BOOKMARK.setValue(self.NODE, "file_logging", __new_filename)

	def on_changed_max_scroll_line (self, spin, parm0=None) :
		__value = spin.get_value()

		self.BOOKMARK.setValue(self.NODE, "max_scroll_line", int(__value))

	def on_set_default_font (self, fontbutton) :
		self.__set_font("set default")

	def on_set_default_color (self, button) :
		self.__set_colors("set default")

	def on_set_default_max_scroll_line (self, button) :
		self.__set_max_scroll_line("set default")

	def on_changed_color (self, colorbutton, which) :
		self.BOOKMARK.setValue( \
			self.NODE, which, func.color_to_str(colorbutton.get_color()))

	def on_changed_background_type (self, radiobutton, which) :
		if which == "transparent" :
			self.XML_BACKGROUND.get_widget("vbox9").set_property("sensitive", True)
			self.XML_BACKGROUND.get_widget("hbox7").set_property("sensitive", False)
		elif which == "image-file" :
			self.XML_BACKGROUND.get_widget("vbox9").set_property("sensitive", True)
			self.XML_BACKGROUND.get_widget("hbox7").set_property("sensitive", True)

			__filename = self.XML_BACKGROUND.get_widget("filechooserbutton1").get_filename()
			if not __filename :
				return
			else :
				self.BOOKMARK.setValue( \
					self.NODE, "vte_background_image_file", __filename)
		else : # none
			self.XML_BACKGROUND.get_widget("hbox7").set_property("sensitive", False)
			self.XML_BACKGROUND.get_widget("vbox9").set_property("sensitive", False)

		self.BOOKMARK.setValue(self.NODE, "vte_background_type", which)

	def on_changed_saturation (self, hscale, parm=None) :
		__value = round(hscale.get_value(), 1)
		self.BOOKMARK.setValue(self.NODE, "vte_background_shade", __value)

	def on_changed_background_image_file (self, chooser) :
		__new_filename = chooser.get_filename()

		if not __new_filename :
			chooser.set_filename(self.DATA["vte_background_image_file"])
			return

		self.BOOKMARK.setValue(self.NODE, "vte_background_type", "image-file")
		self.BOOKMARK.setValue(self.NODE, \
			"vte_background_image_file", __new_filename)

	# MISC HANDLER ########################################################
	def __initialize_vte (self) :
		self.__set_colors()
		self.__set_font()
		self.__initialize_color_button()
		self.__set_background()
		self.__initialize_background()
		self.__set_max_scroll_line()
		self.__set_filelogging()

	def __set_filelogging (self, setDefault=False) :
		if self.DATA["file_logging"].strip() :
			self.XML_ETC.get_widget("entry2").set_text(self.DATA["file_logging"])

		self.XML_ETC.get_widget("checkbutton2").set_active(self.DATA["enable_logging"])
		self.XML_ETC.get_widget("table3").set_property( \
			"sensitive", self.DATA["enable_logging"])

		# enable scrollbar?
		self.XML_ETC.get_widget("checkbutton3").set_active(self.DATA["enable_scrollbar"])

	def __set_max_scroll_line (self, setDefault=False) :
		__spinbutton = self.XML_ETC.get_widget("spinbutton1")
		__spinbutton.set_alignment(1.0)

		if setDefault :
			__value = self.GCONF_CLIENT.getValue("max_scroll_line")
		else :
			__value = self.DATA["max_scroll_line"]

		__spinbutton.set_value(int(__value))

	def __set_font (self, setDefault=False) :

		__checkbutton = self.XML_FONT.get_widget("checkbutton1")
		__fontbutton = self.XML_FONT.get_widget("fontbutton1")

		if setDefault :
			__fontbutton.set_font_name(self.GCONF_CLIENT.getValue("font"))
			__checkbutton.set_active( \
				self.GCONF_CLIENT.getValue("use_system_default_font"))
		else :
			__fontbutton.set_font_name(self.DATA["font"])
			__checkbutton.set_active(self.DATA["use_system_default_font"])

		__fontbutton = self.XML_FONT.get_widget("fontbutton1")
		__box_font = self.XML_FONT.get_widget("hbox3")

		__box_font.set_property("sensitive", not __checkbutton.get_active())

	def __initialize_background (self) :
		# connect handler with event.
		self.XML_BACKGROUND.get_widget("radiobutton1").connect("toggled", \
			self.on_changed_background_type, "none")
		self.XML_BACKGROUND.get_widget("radiobutton2").connect("toggled", \
			self.on_changed_background_type, "image-file")
		self.XML_BACKGROUND.get_widget("radiobutton3").connect("toggled", \
			self.on_changed_background_type, "transparent")

	def __set_background (self) :
		__radio_none					= self.XML_BACKGROUND.get_widget("radiobutton1")
		__radio_background_image	= self.XML_BACKGROUND.get_widget("radiobutton2")
		__radio_transparent			= self.XML_BACKGROUND.get_widget("radiobutton3")

		if self.DATA["vte_background_type"] == "transparent" :
			__radio_transparent.set_active(True)

			self.XML_BACKGROUND.get_widget("hbox7").set_property("sensitive", False)
		elif self.DATA["vte_background_type"] == "image-file" :
			__radio_background_image.set_active(True)
			self.XML_BACKGROUND.get_widget("filechooserbutton3").set_filename( \
				self.DATA["vte_background_image_file"])

			self.XML_BACKGROUND.get_widget("vbox9").set_property("sensitive", False)
		else :
			__radio_none.set_active(True)

			self.XML_BACKGROUND.get_widget("hbox7").set_property("sensitive", False)
			self.XML_BACKGROUND.get_widget("vbox9").set_property("sensitive", False)

		__scale_shade = self.DATA["vte_background_shade"]
		if __scale_shade < 0 or __scale_shade > 1 :
			__scale_shade = 0.2

		# set shade scale.
		self.XML_BACKGROUND.get_widget("hscale1").set_value(__scale_shade)

	def __initialize_color_button (self) :
		self.XML_COLOR.get_widget("colorbutton1").connect("color-set", \
			 self.on_changed_color, "color_foreground")
		self.XML_COLOR.get_widget("colorbutton2").connect("color-set", \
			 self.on_changed_color, "color_background")
		self.XML_COLOR.get_widget("colorbutton3").connect("color-set", \
			 self.on_changed_color, "color_dim")
		self.XML_COLOR.get_widget("colorbutton4").connect("color-set", \
			 self.on_changed_color, "color_highlight")
		self.XML_COLOR.get_widget("colorbutton5").connect("color-set", \
			 self.on_changed_color, "color_cursor")

	def __set_colors (self, set_default=False) :
		__colorbutton_foreground	= self.XML_COLOR.get_widget("colorbutton1")
		__colorbutton_background	= self.XML_COLOR.get_widget("colorbutton2")
		__colorbutton_dim				= self.XML_COLOR.get_widget("colorbutton3")
		__colorbutton_highlight		= self.XML_COLOR.get_widget("colorbutton4")
		__colorbutton_cursor			= self.XML_COLOR.get_widget("colorbutton5")

		if set_default :
			__color_foreground	= \
				func.str_to_color( \
					self.GCONF_CLIENT.getValueByConf("color_foreground"))
			__color_background	= \
				func.str_to_color( \
					self.GCONF_CLIENT.getValueByConf("color_background"))
			__color_dim				= \
				func.str_to_color(self.GCONF_CLIENT.getValueByConf("color_dim"))
			__color_highlight		= \
				func.str_to_color( \
					self.GCONF_CLIENT.getValueByConf("color_highlight"))
			__color_cursor			= \
				func.str_to_color(self.GCONF_CLIENT.getValueByConf("color_cursor"))
		else :
			__color_foreground	= func.str_to_color(self.DATA["color_foreground"])
			__color_background	= func.str_to_color(self.DATA["color_background"])
			__color_dim				= func.str_to_color(self.DATA["color_dim"])
			__color_highlight		= func.str_to_color(self.DATA["color_highlight"])
			__color_cursor			= func.str_to_color(self.DATA["color_cursor"])

		__colorbutton_foreground.set_color(__color_foreground)
		__colorbutton_background.set_color(__color_background)
		__colorbutton_dim.set_color(__color_dim)
		__colorbutton_highlight.set_color(__color_highlight)
		__colorbutton_cursor.set_color(__color_cursor)

		# emit the color-changing signal.
		__colorbutton_foreground.emit("color-set")
		__colorbutton_background.emit("color-set")
		__colorbutton_dim.emit("color-set")
		__colorbutton_highlight.emit("color-set")
		__colorbutton_cursor.emit("color-set")


"""
Description
-----------


ChangeLog
---------


Usage
-----


"""

__author__ =  "Spike^ekipS <spikeekips@gmail.com>"
__version__=  "0.1"
__nonsense__ = ""

__file__ = "__PROPERTY__.py"


