# -*- coding: utf-8 -*-
#	Copyright 2005 Spike^ekipS <spikeekips@gmail.com>
#
#	   This program is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from twisted.conch.error import ConchError
from twisted.conch.client import options, agent, connect, default
from twisted.internet import stdio, defer, utils, protocol, reactor
from twisted.conch.ssh import \
	connection, connection, common, transport, userauth, channel

import sys, os, pickle, struct, base64
import __OS__

class ClientOptions (options.ConchOptions) :

	synopsis = """Usage:   conch --uid <UID filename>
"""

	optParameters = [ \
		["escape", "e", "~"],
		["localforward", "L", None, "listen-port:host:port   Forward local port to remote address"],
		["remoteforward", "R", None, "listen-port:host:port   Forward remote port to local address"],
		["uid", "U", None, "uid for VEE"],
	]

	optFlags = [["null", "n", "Redirect input from /dev/null."],
		["fork", "f", "Fork to background after authentication."],
		["tty", "t", "Tty; allocate a tty even if command is given."],
		["notty", "T", "Do not allocate a tty."],
		["noshell", "N", "Do not execute a shell or command."],
		['subsystem', 's', 'Subsystem/server program to connect to.'], \
	]

	#zsh_altArgDescr = {"foo":"use this description for foo instead"}
	#zsh_multiUse = ["foo", "bar"]
	#zsh_mutuallyExclusive = [("foo", "bar"), ("bar", "baz")]
	#zsh_actions = {"foo":"_files -g "*.foo"", "bar":"(one two three)"}
	zsh_actionDescr = {"localforward":"listen-port:host:port",
					   "remoteforward":"listen-port:host:port"}
	zsh_extras = ["*:command: "]

	localForwards = []
	remoteForwards = []

	def opt_uid (self, uid) :
		if not os.path.isfile(uid) :
			raise RuntimeError, "uid must be given."
		else :
			fd = file(uid, "r")
			__d = pickle.load(fd)
			self["user"] = __d["user"]
			self["host"] = __d["host"]
			self["port"] = __d["port"]

			if __d.has_key("password") :
				self["password"] = __d["password"]

	def opt_escape (self, esc) :
		"Set escape character; ``none"" = disable"
		if esc == "none":
			self["escape"] = None
		elif esc[0] == "^" and len(esc) == 2:
			self["escape"] = chr(ord(esc[1])-64)
		elif len(esc) == 1:
			self["escape"] = esc
		else:
			sys.exit("Bad escape character '%s'." % esc)

	def opt_localforward (self, f) :
		"Forward local port to remote address (lport:host:port)"
		localPort, remoteHost, remotePort = f.split(":") # doesn"t do v6 yet
		localPort = int(localPort)
		remotePort = int(remotePort)
		self.localForwards.append((localPort, (remoteHost, remotePort)))

	def opt_remoteforward (self, f) :
		"""Forward remote port to local address (rport:host:port)"""
		remotePort, connHost, connPort = f.split(":") # doesn"t do v6 yet
		remotePort = int(remotePort)
		connPort = int(connPort)
		self.remoteForwards.append((remotePort, (connHost, connPort)))

	def parseArgs (self, host=None, *command) :
		#self["host"] = host
		self["command"] = " ".join(command)

class ClientOptions_Session (ClientOptions) :

	optParameters = [ \
		["escape", "e", "~"],
		["localforward", "L", None, "listen-port:host:port   Forward local port to remote address"],
		["remoteforward", "R", None, "listen-port:host:port   Forward remote port to local address"],
		["uid", "U", None, "uid for VEE"],
	]

	#optParameters = [
	#	["escape", "e", "~"],
	#	["localforward", "L", None, "listen-port:host:port   Forward local port to remote address"],
	#	["remoteforward", "R", None, "listen-port:host:port   Forward remote port to local address"],
	#	["uid", "U", None, "uid for VEE"],
	#	['buffersize', 'B', 32768, 'Size of the buffer to use for sending/receiving.'],
	#	['batchfile', 'b', None, 'File to read commands from, or \'-\' for stdin.'],
	#	['requests', 'R', 5, 'Number of requests to make before waiting for a reply.'],
	#	['subsystem', 's', 'sftp', 'Subsystem/server program to connect to.'], \
	#]

	optFlags = [["null", "n", "Redirect input from /dev/null."],
		["fork", "f", "Fork to background after authentication."],
		["tty", "t", "Tty; allocate a tty even if command is given."],
		["notty", "T", "Do not allocate a tty."],
		["noshell", "N", "Do not execute a shell or command."],
		['subsystem', 's', 'Subsystem/server program to connect to.'], \
	]

	#zsh_altArgDescr = {"foo":"use this description for foo instead"}
	#zsh_multiUse = ["foo", "bar"]
	#zsh_mutuallyExclusive = [("foo", "bar"), ("bar", "baz")]
	#zsh_actions = {"foo":"_files -g "*.foo"", "bar":"(one two three)"}
	zsh_actionDescr = {"localforward":"listen-port:host:port",
					   "remoteforward":"listen-port:host:port"}
	zsh_extras = ["*:command: "]

	localForwards = []
	remoteForwards = []

	def opt_uid (self, uid) :
		if not os.path.isfile(uid) :
			raise RuntimeError, "uid must be given."
		else :
			fd = file(uid, "r")
			__d = pickle.load(fd)
			self["user"] = __d["user"]
			self["host"] = __d["host"]
			self["port"] = __d["port"]

			if __d.has_key("password") :
				self["password"] = __d["password"]

	def opt_escape (self, esc) :
		"Set escape character; ``none"" = disable"
		if esc == "none":
			self["escape"] = None
		elif esc[0] == "^" and len(esc) == 2:
			self["escape"] = chr(ord(esc[1])-64)
		elif len(esc) == 1:
			self["escape"] = esc
		else:
			sys.exit("Bad escape character '%s'." % esc)

	def opt_localforward (self, f) :
		"Forward local port to remote address (lport:host:port)"
		localPort, remoteHost, remotePort = f.split(":") # doesn"t do v6 yet
		localPort = int(localPort)
		remotePort = int(remotePort)
		self.localForwards.append((localPort, (remoteHost, remotePort)))

	def opt_remoteforward (self, f) :
		"""Forward remote port to local address (rport:host:port)"""
		remotePort, connHost, connPort = f.split(":") # doesn"t do v6 yet
		remotePort = int(remotePort)
		connPort = int(connPort)
		self.remoteForwards.append((remotePort, (connHost, connPort)))

	def parseArgs (self, host=None, *command) :
		#self["host"] = host
		self["command"] = " ".join(command)

class ClientOptions_Sftp (ClientOptions) :

	optParameters = [ \
		["escape", "e", "~"],
		["localforward", "L", None, "listen-port:host:port   Forward local port to remote address"],
		["remoteforward", "R", None, "listen-port:host:port   Forward remote port to local address"],
		["uid", "U", None, "uid for VEE"],
		['subsystem', 's', 'sftp', 'Subsystem/server program to connect to.'], \
		['buffersize', 'B', 2768, 'Size of the buffer to use for sending/receiving.'],
		['requests', 'R', 5, 'Number of requests to make before waiting for a reply.'],
	]

	optFlags = [["null", "n", "Redirect input from /dev/null."],
		["fork", "f", "Fork to background after authentication."],
		["tty", "t", "Tty; allocate a tty even if command is given."],
		["notty", "T", "Do not allocate a tty."],
		["noshell", "N", "Do not execute a shell or command."],
	]

class SSHUserAuthClient_SESSION (default.SSHUserAuthClient) :

	def getPassword (self, prompt = None) :
		return defer.succeed(self.options["password"])

	def getGenericAnswers (self, name, instruction, prompts) :
		responses = []
		try:
			oldout, oldin = sys.stdout, sys.stdin
			sys.stdin = sys.stdout = open('/dev/tty','r+')

			for prompt, echo in prompts:
				if self.options.has_key("password") :
					responses.append(self.options["password"])
				elif echo:
					responses.append(raw_input(prompt))
				else:
					responses.append(getpass.getpass(prompt))

		finally:
			sys.stdout,sys.stdin=oldout,oldin

		return defer.succeed(responses)

class SSHUserAuthClient_SCP(userauth.SSHUserAuthClient):

	WINDOW = None

	def __init__ (self, user, instance, conf, window) :
		userauth.SSHUserAuthClient.__init__ (self, user, instance)
		self.CONF = conf
		self.WINDOW = window
		instance.CONF = conf

	def getPassword(self, prompt = None):
		return defer.succeed(self.CONF["options"]["password"])

	def getGenericAnswers(self, name, instruction, prompts):
		responses = []
		for prompt, echo in prompts:
			if self.CONF["options"].has_key("password") :
				responses.append(self.CONF["options"]["password"])
			elif echo:
				responses.append(raw_input(prompt))
			else:
				responses.append(getpass.getpass(prompt))

		return defer.succeed(responses)

class SSHConnection_SCP (connection.SSHConnection) :
	CONF = None
	WINDOW = None

	def serviceStarted (self) :
		__session = SSHSession_SCP()
		__session.CONF = self.CONF
		__session.WINDOW = self.WINDOW
		self.openChannel(__session)

class SSHSession_SCP (channel.SSHChannel) :

	name = 'session'
	CONF = None
	WINDOW = None

	def channelOpen (self, foo):

		if type(self.conn.CONF["options"]['subsystem']) is str :
			if self.conn.CONF["options"]['subsystem'].startswith('/') :
				request = 'exec'
			else:
				request = 'subsystem'
		else :
			request = 'subsystem'

		d = self.conn.sendRequest(self, request, \
			common.NS(self.conn.CONF["options"]['subsystem']), \
			wantReply=1 \
		)

		d.addCallback(self._cbSubsystem)
		d.addErrback(self._ebExit)

	def _ebExit (self, f) :
		if hasattr(f.value, 'value'):
			s = f.value.value
		else:
			s = str(f)

		#exitStatus = "conch: exiting with error %s" % f
		try:
			self.conn.transport.loseConnection()
		except:
			pass

	def _cbSubsystem (self, result):
		self.client = self.CONF["filetransfer"].getClient()
		self.client.makeConnection(self)

		self.dataReceived = self.client.dataReceived
		self.stdio = sys.stdout

	def extReceived (self, t, data):
		if t==connection.EXTENDED_DATA_STDERR:
			sys.stderr.write(data)
			sys.stderr.flush()

	def eofReceived (self):
		self.stdio.closeStdin()

	def closeReceived (self):
		self.conn.sendClose(self)

	def closed (self):
		pass
		#try :
		#	reactor.stop()
		#except :
		#	pass

	def stopWriting (self):
		#self.stdio.stopReading()
		print "stop writing"

	def startWriting (self):
		print "start writing"
		#self.stdio.startReading()

class SSHClientTransport_SCP (transport.SSHClientTransport) :

	def __init__ (self, conf, window) :
		self.CONF = conf
		self.WINDOW = window

	def receiveError(self, code, desc):
		global exitStatus
		exitStatus = 'conch:\tRemote side disconnected with error code %i\nconch:\treason: %s' % (code, desc)

	def verifyHostKey(self, pubKey, fingerprint):
		d = defer.Deferred()
		d.addCallback(lambda x:defer.succeed(1))
		d.callback(2)
		return d

	def connectionSecure(self):
		if self.CONF["options"]['user']:
			user = self.CONF["options"]['user']
		else:
			user = getpass.getuser()

		__connection = SSHConnection_SCP()
		self.WINDOW.CONNECTION = __connection
		__connection.WINDOW = self.WINDOW

		self.requestService( \
			SSHUserAuthClient_SCP( \
				user, \
				__connection, \
				self.CONF, \
				self.WINDOW, \
			) \
		)

class SSHClientFactory(protocol.ClientFactory):
	noisy = 1
	CONF = None
	WINDOW = None

	def __init__ (self, conf, window) :
		self.CONF = conf
		self.WINDOW = window

	def stopFactory(self):
		print ">> connection stopped."
		#reactor.stop()

	def buildProtocol(self, addr):
		return SSHClientTransport_SCP(self.CONF, self.WINDOW)

def verifyHostKey(transport, host, pubKey, fingerprint):
	goodKey = default.isInKnownHosts(host, pubKey, transport.factory.options)
	if goodKey == 1: # good key
		return defer.succeed(1)
	elif goodKey == 2: # AAHHHHH changed
		return defer.fail(ConchError('changed host key'))
	else:
		oldout, oldin = sys.stdout, sys.stdin
		sys.stdin = sys.stdout = open('/dev/tty','r+')
		if host == transport.transport.getPeer().host:
			khHost = host
		else:
			host = '%s (%s)' % (host,
								transport.transport.getPeer().host)
			khHost = '%s,%s' % (host,
								transport.transport.getPeer().host)
		keyType = common.getNS(pubKey)[0]
		print \
"""The authenticity of host '%s' can't be established.
%s key fingerprint is %s.""" % \
			( \
				host, \
				{'ssh-dss':'DSA', 'ssh-rsa':'RSA'}[keyType], \
				fingerprint \
			)
		try:
			ans = \
				raw_input("Are you sure you want to continue connecting (yes/no)? ")
		except KeyboardInterrupt:
			return defer.fail(ConchError("^C"))
		while ans.lower() not in ('yes', 'no'):
			ans = raw_input("Please type 'yes' or 'no': ")
		sys.stdout,sys.stdin=oldout,oldin
		if ans == "no":
			print "Host key verification failed."
			return defer.fail(ConchError("bad host key"))

		print \
			"Warning: Permanently added '%s' (%s) to the list of known hosts." % \
			(khHost, {"ssh-dss":"DSA", "ssh-rsa":"RSA"}[keyType])

		if not __OS__.is_file(os.path.expanduser("~/.ssh/known_hosts")) :
			known_hosts = open(os.path.expanduser("~/.ssh/known_hosts"), "w")
			known_hosts.close()

		try :
			known_hosts = open(os.path.expanduser("~/.ssh/known_hosts"), "r")
		except IOError :
			pass
		else :
			try :
				known_hosts.seek(-1, 2)
			except IOError :
				__last_line_is = "\n"
			else :
				__last_line_is = known_hosts.read(1)

			known_hosts.close()

			known_hosts = open(os.path.expanduser("~/.ssh/known_hosts"), "a")

			if __last_line_is != '\n':
				known_hosts.write('\n')

			encodedKey = base64.encodestring(pubKey).replace('\n', '')
			known_hosts.write('%s %s %s\n' % (khHost, keyType, encodedKey))
			known_hosts.close()

		return defer.succeed(1)

"""
Description
-----------


ChangeLog
---------


Usage
-----


"""

__author__ =  "Spike^ekipS <spikeekips@gmail.com>"
__version__=  "0.1"
__nonsense__ = ""

__file__ = "__SSHClient__.py"


