# -*- coding: utf-8 -*-
#    Copyright 2005 Spike^ekipS <spikeekips@gmail.com>
#
#       This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import gtk

class Core :

	"""
	The basic treeview class.

	Properties
	===========
		enable-search
		expander-column
		fixed-height-mode
		hadjustment
		headers-clickable
		headers-visible
		hover-expand
		hover-selection
		model
		reorderable
		rules-hint
		search-column
		vadjustment

	PROPERTIES = { \
		"enable-search" : True, \
		"expander-column" : , \
		"fixed-height-mode" : True, \
		"hadjustment" : True, \
		"headers-clickable" : True, \
		"headers-visible" : True, \
		"hover-expand" : True, \
		"hover-selection" : True, \
		"model" : True, \
		"reorderable" : False, \
		"rules-hint" : True, \
		"search-column" : -1, \
		"vadjustment" : , \
	}

	"""

	XML = None
	WIDGET_TREEVIEW = None
	WIDGET_TREESTORE = None
	MODEL = None

	"""
	GTK selection constants are,
		gtk.SELECTION_NONE	No selection allowed.
		gtk.SELECTION_SINGLE	A single selection allowed by clicking.
		gtk.SELECTION_BROWSE	A single selection allowed by browsing with the pointer.
		gtk.SELECTION_MULTIPLE	Multiple items can be selected at once.
		gtk.SELECTION_EXTENDED	Deprecated.
	"""
	SELECTION_MODE = gtk.SELECTION_SINGLE
	SELECTION_CURRENT = None

	PATH_CURRENT = (0,)

	def __init__ (self, xml, treeview_name, properties=dict()) :

		self.XML = xml
		self.WIDGET_TREEVIEW = self.XML.get_widget(treeview_name)

		# set property
		if len(properties) > 0 :
			for i, j in properties.items() :
				self.WIDGET_TREEVIEW.set_property(i, j)

		self.create_frame()
		self.insert_row()

		__selection = self.WIDGET_TREEVIEW.get_selection()
		__selection.set_mode(self.SELECTION_MODE)
		__selection.connect("changed", self.on_change_selection)

		# connect handler
		self.WIDGET_TREEVIEW.connect("row-activated", self.on_row_activated)

	def create_frame (self) : pass
	def insert_row (self) : pass
	def add_row (self, *args) : pass

	###############################################################
	# misc methods
	def set_editable (self, iter, column_number, cell_number) :
		__path = self.get_path(iter)

		__column = self.WIDGET_TREEVIEW.get_column(column_number)
		cell = __column.get_cell_renderers()[cell_number]
		cell.set_property("editable", True)

		self.WIDGET_TREEVIEW.set_cursor(__path, __column, True)

	def clear (self) :
		return self.WIDGET_TREESTORE.clear()

	def append (self, rows, parent=None) :
		return self.WIDGET_TREESTORE.append(parent, rows)

	def prepend (self, rows, parent=None) :
		self.WIDGET_TREESTORE.prepend(parent, rows)

	def set_cursor (self, path) :
		self.WIDGET_TREEVIEW.set_cursor(path)

	def move (self, from_path, to_path) :
		if from_path == to_path : return

		__iter_from = self.WIDGET_TREESTORE.get_iter(from_path)
		if not __iter_from : return
		__iter_to = self.WIDGET_TREESTORE.get_iter(to_path)
		if not __iter_to : return

		self.WIDGET_TREESTORE.move_after(__iter_from, __iter_to)

	def remove (self, **kwargs) :
		"""
		One of path, iter must be given.

		@return : bool()
		"""

		treeiter = None
		if kwargs.has_key("treeiter") : treeiter = kwargs["treeiter"]
		if kwargs.has_key("path") :
			treeiter = self.WIDGET_TREESTORE.get_iter(kwargs["path"])

		if not treeiter : return False

		self.WIDGET_TREESTORE.remove(treeiter)
		#self.WIDGET_TREEVIEW.set_cursor((1,))

		return True

	def get_path (self, iter) :
		return self.WIDGET_TREESTORE.get_path(iter)

	def get_iter (self, path) :
		__treeiter = self.WIDGET_TREESTORE.get_iter(path)

		return __treeiter

	def get_value (self, path, column_number) :
		__treeiter = self.WIDGET_TREESTORE.get_iter(path)
		__value = self.WIDGET_TREESTORE.get_value(__treeiter, column_number)

		return __value

	def set_value (self, path, column_number, value) :
		__iter = self.WIDGET_TREESTORE.get_iter(path)

		if not __iter : return False

		self.WIDGET_TREESTORE.set_value( \
			__iter, \
			column_number, \
			value \
		)

	def get_selected (self) :
		__selection = self.WIDGET_TREEVIEW.get_selection()
		if __selection.get_mode() == \
				gtk.SELECTION_MULTIPLE :
			(__treestore, __treeiter) = \
				__selection.get_selected_rows()
		else :
			(__treestore, __treeiter) = \
				__selection.get_selected()

		return (__treestore, __treeiter, )

	def get_value_selected (self, column_number) :
		(__treestore, __treeiter, ) = self.get_selected()
		if not __treeiter : return

		return self.WIDGET_TREESTORE.get_value(__treeiter, column_number)

	def get_current_path (self, path) :
		return self.PATH_CURRENT

	def set_current_path (self, path) :
		self.PATH_CURRENT = path

	################################################################
	# HANDLER
	def on_change_selection (self, selection) :
		self.SELECTION_CURRENT = selection

	def on_row_activated \
			(self, treeview, path, view_column, user_param1=None) :
		pass


"""
Description
-----------


ChangeLog
---------


Usage
-----


"""

__author__ =  "Spike^ekipS <spikeekips@gmail.com>"
__version__=  "0.1"
__nonsense__ = ""

__file__ = "__TREEVIEW__.py"


