# -*- coding: utf-8 -*-
#    Copyright (C) 2005 Spike^ekipS <spikeekips@gmail.com>
#
#    http://vee.kldp.net
#
#       This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import gtk
import WIDGET, __OS__, Conf, Label

TEMPLATE_TITLE_WITH_LABEL = "%s : %s"
TEMPLATE_TITLE_WITHOUT_LABEL = "%s"

class EGG :

	STATE = None
	STATE_LOCAL = None
	STATE_GLOBAL = None
	WIDGET = None

	SETTING = None
	PROPERTIES = None

	def __init__ (self, state, **kwargs) :
		"""
		* Must be implemented
		"""

		self.STATE = state
		self.STATE_LOCAL = state["local"]
		self.STATE_GLOBAL = state["global"]
		self.GCONF_CLIENT = self.STATE_GLOBAL["gconf-client"]

		self.SETTING = kwargs
		self.PROPERTIES = self.SETTING

		self.make()

	def __del__ (self) :
		self.exit()
		del(self.WIDGET)

	def getProperties (self) :
		return self.PROPERTIES

	def getProperty (self, key) :
		if not self.PROPERTIES.has_key(key) :
			return None
		else :
			return self.PROPERTIES[key]

	def setProperty (self, key, value) :
		self.PROPERTIES.update({key : value})

	def exit (self) :
		"""
		Safely child widget be exited.

		return : None
		"""
		self.WIDGET.destroy()

	def make (self) :
		"""
		* Must be implemented

		Make child widget

		return : None
		"""

		pass

	def getTitle (self) :
		"""
		Get the window title

		return : str
		"""

		try :
			return self.WIDGET.get_window_title()
		except :
			return str()

	def getWidget (self) :
		return self.WIDGET

	def activate_dnd (self) :
		self.WIDGET.connect("drag_data_received", self.on_drag_received)

		self.WIDGET.drag_dest_set(gtk.DEST_DEFAULT_MOTION |
			gtk.DEST_DEFAULT_HIGHLIGHT |
			gtk.DEST_DEFAULT_DROP,
			Conf.DND_TARGET_TABLE, gtk.gdk.ACTION_COPY)

	def on_drag_received \
			(self, widget, context, x, y, selection, targetType, time) :
		return
		#if targetType == Conf.DND_TARGET_COLOR :
		#	pass
		#elif targetType == Conf.DND_TARGET_BGIMAGE :
		#	pass
		#elif targetType == Conf.DND_TARGET_RESET_BG :
		#	pass
		#elif targetType == Conf.DND_TARGET_URI_LIST :
		#	pass
		#elif targetType == Conf.DND_TARGET_MOZ_URL :
		#	pass
		#elif targetType == Conf.DND_TARGET_UTF8_STRING :
		#	pass
		#elif targetType == Conf.DND_TARGET_COMPOUND_TEXT :
		#	pass
		#elif targetType == Conf.DND_TARGET_TEXT_PLAIN :
		#	pass
		#elif targetType == Conf.DND_TARGET_STRING :
		#	pass
		#elif targetType == Conf.DND_TARGET_TEXT_PLAIN :
		#	pass
		#elif targetType == Conf.DND_TARGET_TEXT :
		#	pass
		#else :
		#	pass


class TAB :

	CHILD = None
	NOTEBOOK = None
	STATE = None
	LABEL = None
	SETTING = dict()

	MAIN_WINDOW = None
	EVENT_ID_TITLE_CHANGED = None

	def __init__ (self, state, notebook, child, **kwargs) :

		self.CHILD = child
		self.NOTEBOOK = notebook
		self.STATE = state
		self.SETTING = kwargs

		self.STATE_LOCAL = self.STATE["local"]
		self.STATE_GLOBAL = self.STATE["global"]
		self.GCONF_CLIENT = self.STATE_GLOBAL["gconf-client"]
		self.TAB_LIST = self.STATE_LOCAL["tab-list"]
		self.TAB_HISTORY = self.STATE_LOCAL["tab-history"]
		self.MAIN_WINDOW = self.STATE_LOCAL["main-window"]

		self.make()

	def __del__ (self) :
		self.OBJ_CHILD.__del__()

	def getEgg (self) :
		return self.OBJ_CHILD

	def getEggWidget (self) :
		return self.WIDGET_CHILD

	def createLabel (self, labelname) :

		self.OBJ_LABEL = Label.TabWithClose( \
			labelname=labelname, \
			without_closebutton=False \
		)

		self.OBJ_LABEL.Button.connect("clicked", self.on_remove_tab)

		self.LABEL = self.OBJ_LABEL.getWidget()

	def make (self) :
		self.OBJ_CHILD = self.CHILD(self.STATE, **self.SETTING)
		self.WIDGET_CHILD = self.OBJ_CHILD.getWidget()

		if not isinstance(self.WIDGET_CHILD, gtk.Widget) :
			raise TypeError, "this child widget is not valid gtk widget."
			return

		if not self.SETTING.has_key("label") :
			__labelname = "New Tab"
		else :
			__labelname = self.SETTING["label"]

		self.createLabel(__labelname)

		current_page = self.NOTEBOOK.get_current_page()

		#self.TAB_HISTORY.add(current_page)

		if hasattr(self.OBJ_CHILD, "SIGNAL_REMOVE") :
			self.WIDGET_CHILD.connect( \
				self.OBJ_CHILD.SIGNAL_REMOVE, self.on_remove_tab)

		if hasattr(self.OBJ_CHILD, "SIGNAL_TITLE_CHANGED") :
			self.EVENT_ID_TITLE_CHANGED = \
				self.WIDGET_CHILD.connect( \
					self.OBJ_CHILD.SIGNAL_TITLE_CHANGED, self.on_change_title)

	def setScrolledWindowPolicy (self) :
		self.SCROLLEDWINDOW.set_policy(gtk.POLICY_NEVER, gtk.POLICY_NEVER)

	def attach (self) :
		self.SCROLLEDWINDOW = gtk.ScrolledWindow()

		self.setScrolledWindowPolicy()

		self.SCROLLEDWINDOW.set_border_width(0)
		self.SCROLLEDWINDOW.set_shadow_type(gtk.SHADOW_NONE)
		self.SCROLLEDWINDOW.set_property("can-focus", True)
		self.SCROLLEDWINDOW.set_property("has-focus", True)

		self.SCROLLEDWINDOW.add_with_viewport(self.WIDGET_CHILD)
		self.SCROLLEDWINDOW.show()

		self.NOTEBOOK.append_page(self.SCROLLEDWINDOW, self.LABEL)
		self.WIDGET_CHILD.grab_focus()

		try :
			WIDGET.set_title( \
				self.MAIN_WINDOW, \
				self.GCONF_CLIENT, \
				self.OBJ_CHILD.getTitle()
			)
		except :
			pass

		return True

	def remove_tab (self) :
		current_page = self.NOTEBOOK.get_current_page()
		self.NOTEBOOK.remove_page(current_page)
		self.TAB_LIST.remove(current_page)

		self.NOTEBOOK.emit("change-current-page", Conf.NOTEBOOK_REMOVE_OFFSET)

		self.__del__()

		self.TAB_HISTORY.goPrevious()

	def on_change_title (self, widget) :

		__t = self.OBJ_CHILD.getTitle()
		#self.OBJ_LABEL.set_label(__title)
		if type(__t) not in (str, unicode, ) : return

		if self.SETTING.has_key("label") :
			__title = TEMPLATE_TITLE_WITH_LABEL % (self.SETTING["label"], __t)
		else :
			__title = TEMPLATE_TITLE_WITHOUT_LABEL % __t

		WIDGET.set_title( \
			self.MAIN_WINDOW, \
			self.GCONF_CLIENT, \
			__title, \
		  	"this is file path" \
		)

	def on_remove_tab (self, widget) :
		self.remove_tab()

	def deactivate (self) :
		pass

	def activate (self) :
		pass

"""
Description
-----------


ChangeLog
---------


Usage
-----

"""

__author__ =  "Spike^ekipS <spikeekips@gmail.com>"
__version__=  "0.1"
__nonsense__ = ""

__file__ = "__BASIC__.py"


