# -*- coding: utf-8 -*-
#	 Copyright (C) 2005 Spike^ekipS <spikeekips@gmail.com>
#
#	 http://vee.kldp.net
#
#		 This program is free software; you can redistribute it and/or modify
#	 it under the terms of the GNU General Public License as published by
#	 the Free Software Foundation; either version 2 of the License, or
#	 (at your option) any later version.
#
#	 This program is distributed in the hope that it will be useful,
#	 but WITHOUT ANY WARRANTY; without even the implied warranty of
#	 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	 GNU General Public License for more details.
#
#	 You should have received a copy of the GNU General Public License
#	 along with this program; if not, write to the Free Software
#	 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import os, gtk, vte, pango, time, sre
import child, WIDGET, __BASIC__, GConf, func, i18n, __OS__

_ = i18n.gettext()

DEFAULT_ENVIRONMENT_FOR_SHELL = (\
	"PROMPT_COMMAND='echo -ne \"\\033]0;${PWD}\\007\"'", \
)

class EGG (__BASIC__.EGG) :

	CHILD_TYPE = child.TYPE_VIM
	WIDGET = None

	SIGNAL_REMOVE = "child-exited"
	SIGNAL_TITLE_CHANGED = "window-title-changed"

	# set default value.
	COLOR_BACKGROUND = "black"
	COLOR_FOREGROUND = "ivory"
	COLOR_DIM = "blue"
	COLOR_HIGHLIGHT = "blue"
	COLOR_CURSOR = "red"
	DEFAULT_FONTNAME = "monospace 16"
	EMULATION = "xterm"
	UNIT_FONT_DELM = 1000
	RANGE_FONT_SIZE = (10000, 30000)
	MAX_SCROLL_LINE = 100
	MOUSE_AUTO_HIDE = True

	CURRENT_CURSOR_ROW = 0
	CURRENT_CURSOR_COLUMN = 0
	PREVIOUS_CURSOR_ROW = 0
	PREVIOUS_CURSOR_COLUMN = 0

	def __init__ (self, state, **kwargs) :

		__BASIC__.EGG.__init__(self, state, **kwargs)

		if self.SETTING["enable_logging"] and self.SETTING["file_logging"]:
			self.startLogging(self.SETTING["file_logging"])

	def getWidget (self) :
		return self.WIDGET

	def make (self) :

		self.WIDGET = vte.Terminal()

		self.set_environment()

		self.fork_command()

		self.WIDGET.show()
		self.WIDGET.grab_remove()

		self.activate_dnd()
		#self.attach_popupmenu()

		# connect key-press, key-release event.
		self.WIDGET.connect("key-press-event", self.on_key_press_event)
		self.WIDGET.connect("key-release-event", self.on_key_release_event)
		self.WIDGET.connect("size-allocate", self.on_resize)

	def createDefaultEnvironmentForShell (self) :
		return ( \
			"UID_VEE=%s" % self.STATE_LOCAL["uid"], \
			"TERM=%s" % self.EMULATION, \
		)

	def parse_command (self) :
		__command_line = __OS__.parse_command_line_argument(self.SETTING["command"])

		# default home directory.
		__directory = os.getcwd()

		__argument = (__command_line["command"], ) + __command_line["argument"]
		__environment = \
			self.createDefaultEnvironmentForShell() + \
			__command_line["environment"]

		self.setProperty("executive-command", __command_line["command"])

		return (__command_line["command"], __argument, __environment, __directory, )

	def fork_command (self) :
		(__command, __argument, __environment, __directory, ) = self.parse_command()

		self.WIDGET.fork_command( \
			__command, \
			argv=__argument, \
			envv=__environment, \
			directory=__directory, \
		)

	def changeEnvironment (self) :
		for i in DEFAULT_ENVIRONMENT_FOR_SHELL :
			self.WIDGET.feed_child("export %s\n" % i)

		self.WIDGET.feed_child("clear\n")

	def reset (self, parm1=None, parm2=None, parm3=None) :
		self.WIDGET.reset(True, False)
		self.WIDGET.set_size(80, 24)

	def getTitle (self) :
		__title = self.WIDGET.get_window_title()
		self.setProperty("title", __title)

		return __title

	def decrease_font (self, size=int()) :

		pd = self.WIDGET.get_font()
		__s = pd.get_size() - self.UNIT_FONT_DELM

		if __s not in \
				range(self.RANGE_FONT_SIZE[0], self.RANGE_FONT_SIZE[1]) :
			return

		pd.set_size(__s)
		self.WIDGET.set_font(pd)

		return

	def increase_font (self, size=int()) :

		pd = self.WIDGET.get_font()

		__s = pd.get_size() + self.UNIT_FONT_DELM
		if __s not in \
				range(self.RANGE_FONT_SIZE[0], self.RANGE_FONT_SIZE[1]) :
			return

		pd.set_size(__s)
		self.WIDGET.set_font(pd)

		return

	def set_environment (self, **kwargs) :
		if kwargs : self.SETTING = kwargs
		self.reset()

		self.WIDGET.set_size(80, 24)
		self.WIDGET.set_emulation(self.EMULATION)
		self.WIDGET.set_audible_bell(False)
		self.WIDGET.set_visible_bell(True)
		self.WIDGET.set_scroll_on_output(False)

		self.WIDGET.set_mouse_autohide(self.MOUSE_AUTO_HIDE)
		__max_scroll_line = self.MAX_SCROLL_LINE
		if self.SETTING.has_key("max_scrollline") :
			__max_scroll_line = self.SETTING["max_scrollline"]

		self.WIDGET.set_scrollback_lines(__max_scroll_line)

		self.set_color()
		self.set_background()
		self.set_font()

	def set_font (self, **kwargs) :
		if kwargs : self.SETTING = kwargs

		if not self.SETTING["use_system_default_font"] :
			__font = self.SETTING["font"]
		else :
			__font = self.GCONF_CLIENT.getValueByPath( \
				"/desktop/gnome/interface/monospace_font_name")

		self.WIDGET.set_font_from_string(__font)
		self.reset()

	def set_background (self, **kwargs) :
		if kwargs : self.SETTING = kwargs

		__background_type = None
		__background_shade = None
		__background_image_file = None
		__color_background = self.COLOR_BACKGROUND

		if self.SETTING.has_key("color_background") :
			__color_background = self.SETTING["color_background"]

		if self.SETTING.has_key("vte_background_type") :
			__background_type = self.SETTING["vte_background_type"]
		if self.SETTING.has_key("vte_background_shade") :
			__background_shade = self.SETTING["vte_background_shade"]

		if self.SETTING.has_key("vte_background_image_file") :
			__background_image_file = self.SETTING["vte_background_image_file"]

		# set background
		if __background_type == "transparent" :
			self.WIDGET.set_background_transparent(True)
			self.WIDGET.set_background_tint_color(func.str_to_color(__color_background))
			self.WIDGET.set_background_saturation(__background_shade)
		elif __background_type == "image-file" :
			self.WIDGET.set_background_image_file(__background_image_file)
			self.WIDGET.set_background_saturation(__background_shade)

	def set_color (self, **kwargs) :
		if kwargs : self.SETTING = kwargs

		# initiate terminal color set.
		self.WIDGET.set_default_colors()

		__color_foreground = self.COLOR_FOREGROUND
		if self.SETTING.has_key("color_foreground") :
			__color = self.SETTING["color_foreground"]
		self.WIDGET.set_color_foreground(func.str_to_color(__color))

		__color = self.COLOR_BACKGROUND
		if self.SETTING.has_key("color_background") :
			__color = self.SETTING["color_background"]
		self.WIDGET.set_color_background(func.str_to_color(__color))

		__color = self.COLOR_DIM
		if self.SETTING.has_key("color_dim") :
			__color = self.SETTING["color_dim"]
		self.WIDGET.set_color_dim(func.str_to_color(__color))

		__color = self.COLOR_HIGHLIGHT
		if self.SETTING.has_key("color_highlight") :
			__color = self.SETTING["color_highlight"]
		self.WIDGET.set_color_highlight(func.str_to_color(__color))

		__color = self.COLOR_CURSOR
		if self.SETTING.has_key("color_cursor") :
			__color = self.SETTING["color_cursor"]
		self.WIDGET.set_color_cursor(func.str_to_color(__color))

	def feed (self, str) :
		self.WIDGET.feed_child(str)

	def exit (self) :
		__BASIC__.EGG.exit(self)

	def on_change_color (self, client, parm1, entry, type) :

		__c = self.GCONF_CLIENT.get_value_from_value(entry.get_value())
		__c0 = func.str_to_color(__c)

		try :
			__func = getattr(self.WIDGET, "set_color_%s" % type)
		except AttributeError :
			raise AttributeError, "What type, % is it?" % type
		else :
			__func(__c0)

	IDENTIFIER_LOGGING = None
	FD_LOGGING = None
	def startLogging (self, filename) :
		if not os.path.exists(filename) :
			self.FD_LOGGING = file(filename, "w")
		else :
			self.FD_LOGGING = file(filename, "a")
			self.FD_LOGGING.write("\n")
			self.FD_LOGGING.write( \
				"== (VEE) Truncated at %s.\n" % time.strftime("%c"))

		if not self.IDENTIFIER_LOGGING :
			self.IDENTIFIER_LOGGING = \
				self.WIDGET.connect_after("contents-changed", \
				self.on_content_changed \
			)

	def stopLogging (self) :
		self.WIDGET.disconnect(self.IDENTIFIER_LOGGING)
		self.FD_LOGGING.close()
		self.IDENTIFIER_LOGGING = None

	def __is_selected (self, widget, column, row, data) :
		if row >= self.PREVIOUS_CURSOR_ROW and row < self.CURRENT_CURSOR_ROW :
			return True

		return False

	###########################################################################
	# EVENT HANDLER
	def on_drag_received \
			(self, widget, context, x, y, selection, targetType, time) :

		__path = __OS__.get_filename_from_uri(selection.data)
		self.WIDGET.feed_child(__path)

		return

	def on_content_changed (self, widget) :
		(self.CURRENT_CURSOR_COLUMN, self.CURRENT_CURSOR_ROW, ) = \
			widget.get_cursor_position()

		__text = widget.get_text(self.__is_selected)

		self.FD_LOGGING.write(__text)
		self.FD_LOGGING.flush()

		self.PREVIOUS_CURSOR_COLUMN = self.CURRENT_CURSOR_COLUMN
		self.PREVIOUS_CURSOR_ROW = self.CURRENT_CURSOR_ROW

	def on_resize (self, vte, rectangle) :
		width_char = vte.get_char_width()
		height_char = vte.get_char_height()
		width_vte = rectangle.width
		height_vte = rectangle.height

		columns = width_vte / width_char
		rows = height_vte / height_char

		vte.set_size(columns, rows)

	def on_key_release_event (self, vte, requisition) :
		pass

	def on_key_press_event (self, vte, event) :
		"""
		Prevent from the shortcut key for switching between tabs send to
		child modules.
		"""
		__keyname = func.get_keyname(event)
		__shortcut = \
			self.GCONF_CLIENT.getValue("keyboard_shortcut_modifier_switch_tab")
		__a = sre.compile(sre.escape("%")).split(__shortcut)
		if sre.compile("^%s[0-9][0-9]*$" % __a[0]).findall(__keyname) :
			return True

		return False

class TAB (__BASIC__.TAB) :

	def getSetting (self) :
		"""
			value
			command
			use_system_default_font
			font
			color_foreground
			color_background
			color_dim
			color_highlight
			color_cursor
			max_scroll_line
			vte_background_shade
			vte_background_type
			vte_background_image_file
		"""
		__list_keys_of_default_setting_in_gconf = (
			("use_system_default_font",    "use_system_default_font", ), \
			("font",                       "font", ), \
			("color_foreground",           "color_foreground", ), \
			("color_background",           "color_background", ), \
			("color_dim",                  "color_dim", ), \
			("color_highlight",            "color_highlight", ), \
			("color_cursor",               "color_cursor", ), \
			("max_scroll_line",            "max_scroll_line", ), \
			("vte_background_shade",       "vte_background_shade", ), \
			("vte_background_type",        "vte_background_type", ), \
			("vte_background_image_file",  "vte_background_image_file", ), \
			("enable_scrollbar",           "enable_scrollbar", ), \
		)

		# update setting.
		__setting_default = dict()
		__setting_default.update({"enable_logging" : False, })
		__setting_default.update({"file_logging" : "", })

		for i, j in __list_keys_of_default_setting_in_gconf :
			__values = self.GCONF_CLIENT.getValue(j)
			if __values is None : continue
			__setting_default[i] = __values

		__setting_default.update(self.SETTING)

		if not __setting_default["enable_logging"] and \
				self.GCONF_CLIENT.getValue("enable_logging") :

			__log_directory = os.path.expanduser( \
				self.GCONF_CLIENT.getValue("log_directory") \
			)

			if os.path.isdir(__log_directory) :
				if self.SETTING.has_key("label") :
					__label = self.SETTING["label"]
				else :
					__label = None

				__logfile = os.path.expanduser( \
					os.path.join(__log_directory, __OS__.get_log_filename(__label)) \
				)

				__setting_default["enable_logging"] = True
				__setting_default["file_logging"] = __logfile

		self.SETTING = __setting_default

	def attach (self) :

		######################################################################
		# !! CAUTION !!
		# The vte module in python, creating and operating scrollbar in vte
		# terminal does not work properly.
		#
		#def setScrolledWindowPolicy (self) :
		#	self.SCROLLEDWINDOW.set_policy(gtk.POLICY_NEVER, gtk.POLICY_ALWAYS)
		#	__adjustment = self.WIDGET_CHILD.get_adjustment()
		#
		#	self.SCROLLEDWINDOW.set_vadjustment(__adjustment)

		__box = gtk.HBox()
		__box.pack_start(self.WIDGET_CHILD, True)

		if self.SETTING["enable_scrollbar"] :
			__scrollbar = gtk.VScrollbar()
			__scrollbar.set_adjustment(self.WIDGET_CHILD.get_adjustment())
			__scrollbar.show()
			__box.pack_start(__scrollbar, False)

		__box.show()

		self.NOTEBOOK.append_page(__box, self.LABEL)

		self.WIDGET_CHILD.grab_focus()

		try :
			WIDGET.set_title( \
				self.MAIN_WINDOW, \
				self.GCONF_CLIENT, \
				self.OBJ_CHILD.getTitle()
			)
		except :
			pass

		return True

	EVENT_ID_CONTENTS_CHAGNED = None
	def make (self) :
		self.getSetting()

		__args = __OS__.parse_command_line_argument(self.SETTING["command"])

		if self.SETTING.has_key("label") :
			__labelname = self.SETTING["label"]
		else :
			__labelname = __args["command"]

		self.createLabel(__labelname)

		self.OBJ_CHILD = self.CHILD(self.STATE, **self.SETTING)
		self.OBJ_CHILD.LABEL = self.OBJ_LABEL
		self.OBJ_CHILD.NOTEBOOK = self.NOTEBOOK
		self.WIDGET_CHILD = self.OBJ_CHILD.getWidget()

		self.WIDGET_CHILD.connect( \
			self.OBJ_CHILD.SIGNAL_REMOVE, self.on_remove_tab)
		current_page = self.NOTEBOOK.get_current_page()

		if hasattr(self.OBJ_CHILD, "SIGNAL_TITLE_CHANGED") :
			self.EVENT_ID_TITLE_CHANGED = \
				self.WIDGET_CHILD.connect( \
					self.OBJ_CHILD.SIGNAL_TITLE_CHANGED, self.on_change_title)

	def on_change_title (self, widget) :
		__BASIC__.TAB.on_change_title(self, widget)
		return
		__title = self.OBJ_CHILD.getTitle()
		#self.OBJ_LABEL.set_label(__title)
		if type(__title) not in (str, unicode, ) : return

		WIDGET.set_title( \
			self.MAIN_WINDOW, \
			self.GCONF_CLIENT, \
			__title, \
		)

	def activate (self) :
		return
		if self.EVENT_ID_CONTENTS_CHAGNED :
			self.WIDGET_CHILD.disconnect(self.EVENT_ID_CONTENTS_CHAGNED)
			self.EVENT_ID_CONTENTS_CHAGNED = None

		self.OBJ_LABEL.setActive(False)

	def deactivate (self) :
		return
		# trace the modification of content in VTE.
		self.EVENT_ID_CONTENTS_CHAGNED = \
			self.WIDGET_CHILD.connect("contents-changed", self.on_contents_changed)

	def on_contents_changed (self, vte) :
		self.OBJ_LABEL.setActive()

"""
Description
-----------


ChangeLog
---------


Usage
-----


"""

__author__ =  "Spike^ekipS <spikeekips@gmail.com>"
__version__=  "0.1"
__nonsense__ = ""

__file__ = "__VTE__.py"


