#!/usr/bin/env python
# -*- coding: utf-8 -*-
#    Copyright 2005 Spike^ekipS <spikeekips@gmail.com>
#
#       This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
def usage () :
	print \
"""Usage : %s <glade filename> [po filename] [py filename]
""" % sys.argv[0]

import sys, os, pprint, sre

if len(sys.argv) < 2 :
	print "(WW) No glade filename."
	usage()
	sys.exit(1)

if not os.path.exists(sys.argv[1]) :
	print "(WW) No glade filename."
	usage()
	sys.exit(1)

import xml.dom.ext as ext
from xml.dom import minidom
from xml.xpath import Evaluate

def evaluate_path (dom, xpath) :
	if not hasattr(dom, "documentElement") : return list()

	nodes = Evaluate(xpath, dom.documentElement)

	if len(nodes) < 1 : return list()

	return nodes

def get_element_info (element) :
	widget_name = element.parentNode.getAttribute("id")
	widget_class = element.parentNode.getAttribute("class")
	try :
		__string = element.childNodes[0].nodeValue
	except IndexError :
		__string = ""

	return (str(widget_class), str(widget_name), str(__string), )

# make filename
FILENAME_GLADE = sys.argv[1]
__filename = os.path.splitext(FILENAME_GLADE)[0]

if len(sys.argv) > 2 :
	FILENAME_PO = sys.argv[2]
	FD_PO = file(FILENAME_PO, "a")
else :
	FILENAME_PO = "%s.po" % __filename
	FD_PO = file(FILENAME_PO, "w")

if len(sys.argv) > 3 :
	FILENAME_PY = sys.argv[3]
	FD_PY = file(FILENAME_PY, "a")
else :
	FILENAME_PY = "%s.py" % __filename
	FD_PY = file(FILENAME_PY, "w")


SRE_STRING = sre.compile("[a-z]")

TEMPLATE_PO_WINDOW = \
"""
######################################################################3
# WINDOW : '%s'
"""

TEMPLATE_PO_ONE = \
"""
#Widget is '%(name)s', Widget Type is '%(class)s'.
msgid "%(text)s"
msgstr "%(text)s"
"""

DOM = minidom.parse(FILENAME_GLADE)

__xpath_window = "/glade-interface/widget"
__xpath_translate = "/glade-interface/widget[@id = '%s']//property[@translatable = 'yes']"
nodes = evaluate_path(DOM, __xpath_window)

for i in nodes :
	__window_name = str(i.getAttribute("id"))

	__a = dict()
	__po_string_list = dict()
	__widget_list = dict()
	for j in Evaluate(__xpath_translate % __window_name, i) :
		(__c, __n, __s, ) = get_element_info(j)
		if not __s.strip() : continue
		if not SRE_STRING.findall(__s, sre.IGNORECASE) : continue

		__po_string_list[__s] = (__c, __n, __s, )
		__widget_list[__n] = (__c, __n, __s, )

	FD_PY.write("LABEL_%s = \\\n" % __window_name)
	pp = pprint.PrettyPrinter(indent=4, stream=FD_PY)
	pp.pprint(__widget_list)
	FD_PY.write("\n")

	# CREATE PO FILE ####################################################
	FD_PO.write(TEMPLATE_PO_WINDOW % __window_name)

	for k, m in __po_string_list.items() :
		FD_PO.write(TEMPLATE_PO_ONE % \
			{"name": m[1], "class": m[0], "text": m[2], } \
		)

FD_PO.close()
FD_PY.close()

sys.exit(0)

"""
Description
-----------
This script make po files for glade xml file.


ChangeLog
---------


Usage
-----
python createTranslationForGlade.py <glade filename>

"""

__author__ =  "Spike^ekipS <spikeekips@gmail.com>"
__version__=  "0.1"
__nonsense__ = ""

__file__ = "createTranslationForGlade.py"


